@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3bsetup2.cpp 512488 2006-02-22 17:00:39Z trueg $
- * Copyright (C) 2003-2004 Sebastian Trueg <trueg@k3b.org>
+ * $Id: k3bsetup2.cpp 623771 2007-01-15 13:47:39Z trueg $
+ * Copyright (C) 2003-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,6 +47,8 @@
 #include <k3bdefaultexternalprograms.h>
 #include <k3bglobals.h>
 
+#include <kprocess.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -60,12 +62,16 @@ static bool shouldRunSuidRoot( K3bExternalBin* bin )
   // Since kernel 2.6.8 older cdrecord versions are not able to use the SCSI subsystem when running suid root anymore
   // So for we ignore the suid root issue with kernel >= 2.6.8 and cdrecord < 2.01.01a02
   //
+  // Some kernel version 2.6.16.something again introduced a problem here. Since I do not know the exact version
+  // and a workaround was introduced in cdrecord 2.01.01a05 just use that version as the first for suid root.
+  //
   // Seems as if cdrdao never had problems with suid root...
   //
   
   if( bin->name() == "cdrecord" ) {
     return ( K3b::simpleKernelVersion() < K3bVersion( 2, 6, 8 ) ||
-	     bin->version >= K3bVersion( 2, 1, 1, "a02" ) );
+	     bin->version >= K3bVersion( 2, 1, 1, "a05" ) ||
+	     bin->hasFeature( "wodim" ) );
   }
   else if( bin->name() == "cdrdao" ) {
     return true;
@@ -73,8 +79,10 @@ static bool shouldRunSuidRoot( K3bExternalBin* bin )
   else if( bin->name() == "growisofs" ) {
     //
     // starting with 6.0 growiofs raises it's priority using nice(-20)
-    //
-    return bin->version >= K3bVersion( 6, 0 );
+    // BUT: newer kernels have ridiculously low default memorylocked resource limit, which prevents privileged 
+    // users from starting growisofs 6.0 with "unable to anonymously mmap 33554432: Resource temporarily unavailable" 
+    // error message. Until Andy releases a version including a workaround we simply never configure growisofs suid root
+    return false; // bin->version >= K3bVersion( 6, 0 );
   }
   else
     return false;
@@ -106,10 +114,10 @@ K3bSetup2::K3bSetup2( QWidget *parent, const char *, const QStringList& )
   d = new Private();
   d->config = new KConfig( "k3bsetup2rc" );
 
-  m_aboutData = new KAboutData(I18N_NOOP("k3bsetup2"),
-			       I18N_NOOP("K3bSetup 2"),
+  m_aboutData = new KAboutData("k3bsetup2",
+			       "K3bSetup 2",
 			       0, 0, KAboutData::License_GPL,
-			       I18N_NOOP("(C) 2003-2004 Sebastian Trueg"));
+			       "(C) 2003-2007 Sebastian Trueg");
   m_aboutData->addAuthor("Sebastian Trueg", 0, "trueg@k3b.org");
 
   setButtons( KCModule::Apply|KCModule::Cancel|KCModule::Ok|KCModule::Default );
@@ -120,7 +128,7 @@ K3bSetup2::K3bSetup2( QWidget *parent, const char *, const QStringList& )
   box->setSpacing( KDialog::spacingHint() );
 
   KTextEdit* label = new KTextEdit( this );
-  label->setText( "<h2>" + i18n("K3bSetup") + "</h2>"
+  label->setText( "<h2>K3bSetup</h2>"
 		  + i18n("<p>This simple setup assistant is able to set the permissions needed by K3b in order to "
 			 "burn CDs and DVDs. "
 			 "<p>It does not take things like devfs or resmgr into account. In most cases this is not a "
@@ -289,7 +297,7 @@ void K3bSetup2::updateDevices()
     if( !device->genericDevice().isEmpty() ) {
       QCheckListItem* item = createDeviceItem( device->genericDevice() );
       item->setOn( checkMap.contains(device->genericDevice()) ? checkMap[device->genericDevice()] : true );
-      item->setText( 0, device->vendor() + " " + device->description() );
+      item->setText( 0, device->vendor() + " " + device->description() + " (" + i18n("Generic SCSI Device") + ")" );
     }
   }
 }
@@ -439,30 +447,37 @@ void K3bSetup2::save()
 
       K3bExternalBin* bin = d->listBinMap[checkItem];
 
+      KProcess cleanupProc;
+      cleanupProc << "/usr/sbin/dpkg-statoverride" << "--quiet" << "--remove";
+      cleanupProc << QFile::encodeName(bin->path);
+      if (!cleanupProc.start(KProcess::Block))
+	success = false;
+
       if( w->m_checkUseBurningGroup->isChecked() ) {
-	if( ::chown( QFile::encodeName(bin->path), (gid_t)0, g->gr_gid ) )
-	  success = false;
+	KProcess proc;
+	proc << "/usr/sbin/dpkg-statoverride" << "--quiet" << "--update" << "--add" << "#0" << "#";
+        proc << g->gr_gid;
 
-	int perm = 0;
 	if( shouldRunSuidRoot( bin ) )
-	  perm = S_ISUID|S_IRWXU|S_IXGRP;
+	  proc << "4710";
 	else
-	  perm = S_IRWXU|S_IXGRP|S_IRGRP;
+	  proc << "0750";
 
-	if( ::chmod( QFile::encodeName(bin->path), perm ) )
+	proc << QFile::encodeName(bin->path);
+	if (!proc.start(KProcess::Block))
 	  success = false;
       }
       else {
-	if( ::chown( QFile::encodeName(bin->path), 0, 0 ) )
-	  success = false;
+	KProcess proc;
+	proc << "/usr/sbin/dpkg-statoverride" << "--quiet" << "--update" << "--add" << "#0" << "#0";
 
-	int perm = 0;
 	if( shouldRunSuidRoot( bin ) )
-	  perm = S_ISUID|S_IRWXU|S_IXGRP|S_IXOTH;
+	  proc << "4711";
 	else
-	  perm = S_IRWXU|S_IXGRP|S_IRGRP|S_IXOTH|S_IROTH;
+	  proc << "0755";
 
-	if( ::chmod( QFile::encodeName(bin->path), perm ) )
+	proc << QFile::encodeName(bin->path);
+	if (!proc.start(KProcess::Block))
 	  success = false;
       }
     }
