@@ -1,10 +1,10 @@
 /* 
  *
- * $Id: k3bdefaultexternalprograms.cpp 471081 2005-10-16 10:55:47Z trueg $
- * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
+ * $Id: k3bdefaultexternalprograms.cpp 641682 2007-03-12 09:49:33Z trueg $
+ * Copyright (C) 2003-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <k3bglobals.h>
 
 #include <qfile.h>
+#include <qdir.h>
 #include <qfileinfo.h>
 #include <qobject.h>
 #include <qregexp.h>
@@ -48,6 +49,7 @@ void K3b::addDefaultPrograms( K3bExternalBinManager* m )
 void K3b::addTranscodePrograms( K3bExternalBinManager* m )
 {
   static const char* transcodeTools[] =  { "transcode",
+					   0, // K3b 1.0 only uses the transcode binary
 					   "tcprobe",
 					   "tccat",
 					   "tcscan",
@@ -80,69 +82,93 @@ K3bCdrecordProgram::K3bCdrecordProgram( bool dvdPro )
 }
 
 
+//
+// This is a hack for Debian based systems which use
+// a wrapper cdrecord script to call cdrecord.mmap or cdrecord.shm
+// depending on the kernel version.
+// For 2.0.x and 2.2.x kernels the shm version is used. In all
+// other cases it's the mmap version.
+//
+// But since it may be that someone manually installed cdrecord 
+// replacing the wrapper we check if cdrecord is a script.
+//
+static QString& debianWeirdnessHack( QString& path )
+{
+  if( QFile::exists( path + ".mmap" ) ) {
+    kdDebug() << "(K3bCdrecordProgram) checking for Debian cdrecord wrapper script." << endl;
+    if( QFileInfo( path ).size() < 1024 ) {
+      kdDebug() << "(K3bCdrecordProgram) Debian Wrapper script size fits. Checking file." << endl;
+      QFile f( path );
+      f.open( IO_ReadOnly );
+      QString s = QTextStream( &f ).read();
+      if( s.contains( "cdrecord.mmap" ) && s.contains( "cdrecord.shm" ) ) {
+	kdDebug() << "(K3bCdrecordProgram) Found Debian Wrapper script." << endl;
+	QString ext;
+	if( K3b::kernelVersion().versionString().left(3) > "2.2" )
+	  ext = ".mmap";
+	else
+	  ext = ".shm";
+	
+	kdDebug() << "(K3bCdrecordProgram) Using cdrecord" << ext << endl;
+	
+	path += ext;
+      }
+    }
+  }
+
+  return path;
+}
+
+
 bool K3bCdrecordProgram::scan( const QString& p )
 {
   if( p.isEmpty() )
     return false;
 
+  bool wodim = false;
+  bool cdrskin = false;
   QString path = p;
   QFileInfo fi( path );
   if( fi.isDir() ) {
     if( path[path.length()-1] != '/' )
       path.append("/");
-    path.append("cdrecord");
 
-    //
-    // This is a hack for Debian based systems which use
-    // a wrapper cdrecord script to call cdrecord.mmap or cdrecord.shm
-    // depending on the kernel version.
-    // For 2.0.x and 2.2.x kernels the shm version is used. In all
-    // other cases it's the mmap version.
-    //
-    // But since it may be that someone manually installed cdrecord 
-    // replacing the wrapper we check if cdrecord is a script.
-    //
-    if( QFile::exists( path + ".mmap" ) ) {
-      kdDebug() << "(K3bCdrecordProgram) checking for Debian cdrecord wrapper script." << endl;
-      if( QFileInfo( path ).size() < 1024 ) {
-	kdDebug() << "(K3bCdrecordProgram) Debian Wrapper script size fits. Checking file." << endl;
-	QFile f( path );
-	f.open( IO_ReadOnly );
-	QString s = QTextStream( &f ).read();
-	if( s.contains( "cdrecord.mmap" ) && s.contains( "cdrecord.shm" ) ) {
-	  kdDebug() << "(K3bCdrecordProgram) Found Debian Wrapper script." << endl;
-	  QString ext;
-	  if( K3b::kernelVersion().versionString().left(3) > "2.2" )
-	    ext = ".mmap";
-	  else
-	    ext = ".shm";
-
-	  kdDebug() << "(K3bCdrecordProgram) Using cdrecord" << ext << endl;
-
-	  path += ext;
-	}
-      }
+    if( QFile::exists( path + "cdrskin" ) ) {
+      cdrskin = true;
+      path += "cdrskin";
     }
+    else if( QFile::exists( path + "wodim" ) ) {
+      wodim = true;
+      path += "wodim";
+    }
+    else if( QFile::exists( path + "cdrecord" ) ) {
+      path += "cdrecord";
+    }
+    else
+      return false;
   }
 
-  if( !QFile::exists( path ) )
-    return false;
+  debianWeirdnessHack( path );
 
   K3bExternalBin* bin = 0;
 
   // probe version
   KProcess vp;
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
 
   vp << path << "-version";
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
     int pos = -1;
-    if( m_dvdPro ) {
+    if( wodim ) {
+      pos = out.output().find( "Wodim" );
+    }
+    else if( m_dvdPro ) {
       pos = out.output().find( "Cdrecord-ProDVD" );
     }
     else {
       pos = out.output().find( "Cdrecord" );
     }
+
     if( pos < 0 )
       return false;
 
@@ -150,7 +176,7 @@ bool K3bCdrecordProgram::scan( const QString& p )
     if( pos < 0 )
       return false;
 
-    int endPos = out.output().find( ' ', pos+1 );
+    int endPos = out.output().find( QRegExp("\\s"), pos+1 );
     if( endPos < 0 )
       return false;
 
@@ -158,9 +184,14 @@ bool K3bCdrecordProgram::scan( const QString& p )
     bin->path = path;
     bin->version = out.output().mid( pos, endPos-pos );
 
+    if( wodim )
+      bin->addFeature( "wodim" );
+
     pos = out.output().find( "Copyright") + 14;
     endPos = out.output().find( "\n", pos );
-    bin->copyright = out.output().mid( pos, endPos-pos ).stripWhiteSpace();
+
+    // cdrecord does not use local encoding for the copyright statement but plain latin1
+    bin->copyright = QString::fromLatin1( out.output().mid( pos, endPos-pos ).local8Bit() ).stripWhiteSpace();
   }
   else {
     kdDebug() << "(K3bCdrecordProgram) could not start " << path << endl;
@@ -188,7 +219,7 @@ bool K3bCdrecordProgram::scan( const QString& p )
     if( out.output().contains( "-tao" ) )
       bin->addFeature( "tao" );
     if( out.output().contains( "cuefile=" ) && 
-	bin->version > K3bVersion( 2, 1, -1, "a14") ) // cuefile handling was still buggy in a14
+	( wodim || bin->version > K3bVersion( 2, 1, -1, "a14") ) ) // cuefile handling was still buggy in a14
       bin->addFeature( "cuefile" );
 
     // new mode 2 options since cdrecord 2.01a12
@@ -197,7 +228,8 @@ bool K3bCdrecordProgram::scan( const QString& p )
     // two checks)
     // and the version check does not handle versions like 2.01-dvd properly
     if( out.output().contains( "-xamix" ) ||
-	bin->version >= K3bVersion( 2, 1, -1, "a12" ) )
+	bin->version >= K3bVersion( 2, 1, -1, "a12" ) || 
+	wodim )
       bin->addFeature( "xamix" );
    
     // check if we run cdrecord as root
@@ -213,95 +245,25 @@ bool K3bCdrecordProgram::scan( const QString& p )
     return false;
   }
 
+  if( bin->version < K3bVersion( 2, 0 ) && !wodim )
+    bin->addFeature( "outdated" );
+
   // FIXME: are these version correct?
-  if( bin->version >= K3bVersion("1.11a38") )
+  if( bin->version >= K3bVersion("1.11a38") || wodim )
     bin->addFeature( "plain-atapi" );
-  if( bin->version > K3bVersion("1.11a17") )
+  if( bin->version > K3bVersion("1.11a17") || wodim )
     bin->addFeature( "hacked-atapi" );
 
-  if( bin->version >= K3bVersion( 2, 1, 1, "a02" ) )
+  if( bin->version >= K3bVersion( 2, 1, 1, "a02" ) || wodim )
     bin->addFeature( "short-track-raw" );
 
-  addBin( bin );
-  return true;
-}
+  if( bin->version >= K3bVersion( 2, 1, -1, "a13" ) || wodim )
+    bin->addFeature( "audio-stdin" );
 
-
-K3bDvdrecordProgram::K3bDvdrecordProgram()
-  : K3bExternalProgram( "dvdrecord" )
-{
-}
-
-
-bool K3bDvdrecordProgram::scan( const QString& p )
-{
-  if( p.isEmpty() )
-    return false;
-
-  QString path = p;
-  QFileInfo fi( path );
-  if( fi.isDir() ) {
-    if( path[path.length()-1] != '/' )
-      path.append("/");
-    path.append("dvdrecord");
-  }
-
-  if( !QFile::exists( path ) )
-    return false;
-
-  K3bExternalBin* bin = 0;
-
-  // probe version
-  KProcess vp;
-  K3bProcess::OutputCollector out( &vp );
-
-  vp << path << "-version";
-  if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
-    int pos = out.output().find( "dvdrtools" );
-    if( pos < 0 )
-      return false;
-
-    pos = out.output().find( QRegExp("[0-9]"), pos );
-    if( pos < 0 )
-      return false;
-
-    int endPos = out.output().find( "\n", pos );
-    if( endPos < 0 )
-      return false;
-
-    bin = new K3bExternalBin( this );
-    bin->path = path;
-    bin->version = out.output().mid( pos, endPos-pos );
-  }
-  else {
-    kdDebug() << "(K3bDvdrecordProgram) could not start " << path << endl;
-    return false;
-  }
-
-
-
-  // probe features
-  KProcess fp;
-  out.setProcess( &fp );
-  fp << path << "-help";
-  if( fp.start( KProcess::Block, KProcess::AllOutput ) ) {
-    if( out.output().contains( "-delay" ) )
-      bin->addFeature( "delay" );
-    if( out.output().contains( "-overburn" ) )
-      bin->addFeature( "overburn" );
-    
-    // check if we run cdrecord as root
-    struct stat s;
-    if( !::stat( QFile::encodeName(path), &s ) ) {
-      if( (s.st_mode & S_ISUID) && s.st_uid == 0 )
-	bin->addFeature( "suidroot" );
-    }
-  }
-  else {
-    kdDebug() << "(K3bDvdrecordProgram) could not start " << bin->path << endl;
-    delete bin;
-    return false;
-  }
+  if( bin->version >= K3bVersion( "1.11a02" ) || wodim )
+    bin->addFeature( "burnfree" );
+  else
+    bin->addFeature( "burnproof" );
 
   addBin( bin );
   return true;
@@ -319,25 +281,37 @@ bool K3bMkisofsProgram::scan( const QString& p )
   if( p.isEmpty() )
     return false;
 
+  bool genisoimage = false;
   QString path = p;
   QFileInfo fi( path );
   if( fi.isDir() ) {
     if( path[path.length()-1] != '/' )
       path.append("/");
-    path.append("mkisofs");
-  }
 
-  if( !QFile::exists( path ) )
-    return false;
+    if( QFile::exists( path + "genisoimage" ) ) {
+      genisoimage = true;
+      path += "genisoimage";
+    }
+    else if( QFile::exists( path + "mkisofs" ) ) {
+      path += "mkisofs";
+    }
+    else
+      return false;
+  }
 
   K3bExternalBin* bin = 0;
 
   // probe version
   KProcess vp;
   vp << path << "-version";
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
-    int pos = out.output().find( "mkisofs" );
+    int pos = -1;
+    if( genisoimage )
+      pos = out.output().find( "genisoimage" );
+    else
+      pos = out.output().find( "mkisofs" );
+
     if( pos < 0 )
       return false;
 
@@ -352,6 +326,9 @@ bool K3bMkisofsProgram::scan( const QString& p )
     bin = new K3bExternalBin( this );
     bin->path = path;
     bin->version = out.output().mid( pos, endPos-pos );
+
+    if( genisoimage )
+      bin->addFeature( "genisoimage" );
   }
   else {
     kdDebug() << "(K3bMkisofsProgram) could not start " << path << endl;
@@ -389,6 +366,12 @@ bool K3bMkisofsProgram::scan( const QString& p )
     return false;
   }
 
+  if( bin->version < K3bVersion( 1, 14) && !genisoimage )
+    bin->addFeature( "outdated" );
+
+  if( bin->version >= K3bVersion( 1, 15, -1, "a40" ) || genisoimage )
+    bin->addFeature( "backslashed_filenames" );
+
   addBin(bin);
   return true;
 }
@@ -404,12 +387,22 @@ bool K3bReadcdProgram::scan( const QString& p )
   if( p.isEmpty() )
     return false;
 
+  bool readom = false;
   QString path = p;
   QFileInfo fi( path );
   if( fi.isDir() ) {
     if( path[path.length()-1] != '/' )
       path.append("/");
-    path.append("readcd");
+
+    if( QFile::exists( path + "readom" ) ) {
+      readom = true;
+      path += "readom";
+    }
+    else if( QFile::exists( path + "readcd" ) ) {
+      path += "readcd";
+    }
+    else
+      return false;
   }
 
   if( !QFile::exists( path ) )
@@ -420,9 +413,13 @@ bool K3bReadcdProgram::scan( const QString& p )
   // probe version
   KProcess vp;
   vp << path << "-version";
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
-    int pos = out.output().find( "readcd" );
+    int pos = -1;
+    if( readom )
+      pos = out.output().find( "readom" );
+    else
+      pos = out.output().find( "readcd" );
     if( pos < 0 )
       return false;
 
@@ -437,6 +434,9 @@ bool K3bReadcdProgram::scan( const QString& p )
     bin = new K3bExternalBin( this );
     bin->path = path;
     bin->version = out.output().mid( pos, endPos-pos );
+
+    if( readom )
+      bin->addFeature( "readom" );
   }
   else {
     kdDebug() << "(K3bMkisofsProgram) could not start " << path << endl;
@@ -468,9 +468,9 @@ bool K3bReadcdProgram::scan( const QString& p )
 
 
   // FIXME: are these version correct?
-  if( bin->version >= K3bVersion("1.11a38") )
+  if( bin->version >= K3bVersion("1.11a38") || readom )
     bin->addFeature( "plain-atapi" );
-  if( bin->version > K3bVersion("1.11a17") )
+  if( bin->version > K3bVersion("1.11a17") || readom )
     bin->addFeature( "hacked-atapi" );
 
   addBin(bin);
@@ -504,7 +504,7 @@ bool K3bCdrdaoProgram::scan( const QString& p )
   // probe version
   KProcess vp;
   vp << path ;
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
     int pos = out.output().find( "Cdrdao version" );
     if( pos < 0 )
@@ -591,17 +591,18 @@ bool K3bTranscodeProgram::scan( const QString& p )
   QString path = p;
   if( path[path.length()-1] != '/' )
     path.append("/");
-  path.append(m_transcodeProgram);
 
-  if( !QFile::exists( path ) )
+  QString appPath = path + m_transcodeProgram;
+
+  if( !QFile::exists( appPath ) )
     return false;
 
   K3bExternalBin* bin = 0;
 
   // probe version
   KProcess vp;
-  vp << path ;
-  K3bProcess::OutputCollector out( &vp );
+  vp << appPath << "-v";
+  K3bProcessOutputCollector out( &vp );
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
     int pos = out.output().find( "transcode v" );
     if( pos < 0 )
@@ -614,14 +615,34 @@ bool K3bTranscodeProgram::scan( const QString& p )
       return false;
 
     bin = new K3bExternalBin( this );
-    bin->path = path;
+    bin->path = appPath;
     bin->version = out.output().mid( pos, endPos-pos );
   }
   else {
-    kdDebug() << "(K3bTranscodeProgram) could not start " << path << endl;
+    kdDebug() << "(K3bTranscodeProgram) could not start " << appPath << endl;
     return false;
   }
 
+  //
+  // Check features
+  //
+  QString modInfoBin = path + "tcmodinfo";
+  KProcess modp;
+  modp << modInfoBin << "-p";
+  out.setProcess( &modp );
+  if( modp.start( KProcess::Block, KProcess::AllOutput ) ) {
+    QString modPath = out.output().stripWhiteSpace();
+    QDir modDir( modPath );
+    if( !modDir.entryList( "*export_xvid*", QDir::Files ).isEmpty() )
+      bin->addFeature( "xvid" );
+    if( !modDir.entryList( "*export_lame*", QDir::Files ).isEmpty() )
+      bin->addFeature( "lame" );
+    if( !modDir.entryList( "*export_ffmpeg*", QDir::Files ).isEmpty() )
+      bin->addFeature( "ffmpeg" );
+    if( !modDir.entryList( "*export_ac3*", QDir::Files ).isEmpty() )
+      bin->addFeature( "ac3" );
+  }
+
   addBin(bin);
   return true;
 }
@@ -655,7 +676,7 @@ bool K3bVcdbuilderProgram::scan( const QString& p )
   // probe version
   KProcess vp;
   vp << path << "-V";
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
     int pos = out.output().find( "GNU VCDImager" );
     if( pos < 0 )
@@ -686,7 +707,7 @@ bool K3bVcdbuilderProgram::scan( const QString& p )
 
 
 K3bNormalizeProgram::K3bNormalizeProgram()
-  : K3bExternalProgram( "normalize" )
+  : K3bExternalProgram( "normalize-audio" )
 {
 }
 
@@ -701,7 +722,7 @@ bool K3bNormalizeProgram::scan( const QString& p )
   if( fi.isDir() ) {
     if( path[path.length()-1] != '/' )
       path.append("/");
-    path.append("normalize");
+    path.append("normalize-audio");
   }
 
   if( !QFile::exists( path ) )
@@ -711,7 +732,7 @@ bool K3bNormalizeProgram::scan( const QString& p )
 
   // probe version
   KProcess vp;
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
 
   vp << path << "--version";
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
@@ -770,7 +791,7 @@ bool K3bGrowisofsProgram::scan( const QString& p )
 
   // probe version
   KProcess vp;
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
 
   vp << path << "-version";
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
@@ -835,13 +856,13 @@ bool K3bDvdformatProgram::scan( const QString& p )
 
   // probe version
   KProcess vp;
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
 
   vp << path;
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
     // different locales make searching for the +- char difficult
     // so we simply ignore it.
-    int pos = out.output().find( QRegExp("DVD.*RW(/-RAM)? format utility") );
+    int pos = out.output().find( QRegExp("DVD.*RAM format utility") );
     if( pos < 0 )
       return false;
 
@@ -905,7 +926,7 @@ bool K3bDvdBooktypeProgram::scan( const QString& p )
 
   // probe version
   KProcess vp;
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
 
   vp << path;
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
@@ -954,7 +975,7 @@ bool K3bCdda2wavProgram::scan( const QString& p )
 
   // probe version
   KProcess vp;
-  K3bProcess::OutputCollector out( &vp );
+  K3bProcessOutputCollector out( &vp );
 
   vp << path << "-h";
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
