@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3bdvdcopyjob.cpp 526893 2006-04-05 21:23:45Z trueg $
+ * $Id: k3bdvdcopyjob.cpp 627609 2007-01-27 11:22:24Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,11 +28,13 @@
 #include <k3bgrowisofswriter.h>
 #include <k3bversion.h>
 #include <k3biso9660.h>
+#include <k3bfilesplitter.h>
+#include <k3bchecksumpipe.h>
+#include <k3bverificationjob.h>
 
 #include <kdebug.h>
 #include <klocale.h>
 #include <kio/global.h>
-#include <kmessagebox.h>
 
 #include <qfile.h>
 #include <qfileinfo.h>
@@ -49,7 +51,10 @@ public:
       writerJob(0),
       readcdReader(0),
       dataTrackReader(0),
-      usedWritingMode(0) {
+      verificationJob(0),
+      usedWritingMode(0),
+      verifyData(false) {
+    outPipe.readFromIODevice( &imageFile );
   }
 
   int doneCopies;
@@ -62,12 +67,19 @@ public:
   K3bGrowisofsWriter* writerJob;
   K3bReadcdReader* readcdReader;
   K3bDataTrackReader* dataTrackReader;
+  K3bVerificationJob* verificationJob;
 
   K3bDevice::DiskInfo sourceDiskInfo;
 
   K3b::Msf lastSector;
 
   int usedWritingMode;
+
+  K3bFileSplitter imageFile;
+  K3bChecksumPipe inPipe;
+  K3bActivePipe outPipe;
+
+  bool verifyData;
 };
 
 
@@ -97,13 +109,15 @@ K3bDvdCopyJob::~K3bDvdCopyJob()
 
 void K3bDvdCopyJob::start()
 {
-  emit started();
+  jobStarted();
   emit burning(false);
 
   d->canceled = false;
   d->running = true;
   d->readerRunning = d->writerRunning = false;
 
+  emit newTask( i18n("Checking Source Medium") );
+
   if( m_onTheFly && 
       k3bcore->externalBinManager()->binObject( "growisofs" )->version < K3bVersion( 5, 12 ) ) {
     m_onTheFly = false;
@@ -112,8 +126,19 @@ void K3bDvdCopyJob::start()
     emit infoMessage( i18n("Disabling on-the-fly writing."), INFO );
   }
 
-  emit infoMessage( i18n("Checking source media") + "...", INFO );
-  emit newSubTask( i18n("Checking source media") );
+  emit newSubTask( i18n("Waiting for source medium") );
+
+  // wait for a source disk
+  if( waitForMedia( m_readerDevice,
+		    K3bDevice::STATE_COMPLETE|K3bDevice::STATE_INCOMPLETE,
+		    K3bDevice::MEDIA_WRITABLE_DVD|K3bDevice::MEDIA_DVD_ROM ) < 0 ) {
+    emit canceled();
+    d->running = false;
+    jobFinished( false );
+    return;
+  }
+
+  emit newSubTask( i18n("Checking source medium") );
 
   connect( K3bDevice::sendCommand( K3bDevice::DeviceHandler::DISKINFO, m_readerDevice ),
            SIGNAL(finished(K3bDevice::DeviceHandler*)),
@@ -126,15 +151,15 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 {
   if( d->canceled ) {
     emit canceled();
-    emit finished(false);
+    jobFinished(false);
     d->running = false;
   }
 
   d->sourceDiskInfo = dh->diskInfo();
 
   if( dh->diskInfo().empty() || dh->diskInfo().diskState() == K3bDevice::STATE_NO_MEDIA ) {
-    emit infoMessage( i18n("No source media found."), ERROR );
-    emit finished(false);
+    emit infoMessage( i18n("No source medium found."), ERROR );
+    jobFinished(false);
     d->running = false;
   }
   else {
@@ -148,6 +173,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 	kdDebug() << "(K3bDvdCopyJob) dvdcss_open(" << m_readerDevice->blockDeviceName() << ") = "
 		  << libcss->open(m_readerDevice) << endl;
 	haveLibdvdcss = true;
+
 	delete libcss;
       }
       else
@@ -156,7 +182,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
       if( !haveLibdvdcss ) {
 	emit infoMessage( i18n("Cannot copy encrypted DVDs."), ERROR );
 	d->running = false;
-	emit finished( false );
+	jobFinished( false );
 	return;
       }
     }
@@ -192,7 +218,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 	  if( !(m_writerDevice->type() & (K3bDevice::DEVICE_DVD_R_DL|K3bDevice::DEVICE_DVD_PLUS_R_DL)) ) {
 	    emit infoMessage( i18n("The writer does not support writing Double Layer DVD."), ERROR );
 	    d->running = false;
-	    emit finished(false);
+	    jobFinished(false);
 	    return;
 	  }
 	  // FIXME: check for growisofs 5.22 (or whatever version is needed) for DVD-R DL
@@ -200,7 +226,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 		   k3bcore->externalBinManager()->binObject( "growisofs" )->version < K3bVersion( 5, 20 ) ) {
 	    emit infoMessage( i18n("Growisofs >= 5.20 is needed to write Double Layer DVD+R."), ERROR );
 	    d->running = false;
-	    emit finished(false);
+	    jobFinished(false);
 	    return;
 	  }
 	}
@@ -214,7 +240,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
       if( dh->diskInfo().numSessions() > 1 ) {
 	emit infoMessage( i18n("K3b does not support copying multi-session DVDs."), ERROR );
 	d->running = false;
-	emit finished(false);
+	jobFinished(false);
 	return;
       }
 
@@ -224,7 +250,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
       if( m_writingMode != K3b::DAO || !m_onTheFly || m_onlyCreateImage ||
 	  ( k3bcore->externalBinManager()->binObject( "growisofs" ) && 
 	    k3bcore->externalBinManager()->binObject( "growisofs" )->version >= K3bVersion( 5, 15, -1 ) ) ) {
-	d->lastSector = dh->toc()[0].lastSector();
+	d->lastSector = dh->toc().lastSector();
 	break;
       }
 
@@ -242,7 +268,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 	}
 	else {
 	  emit infoMessage( i18n("Unable to determine the ISO9660 filesystem size."), ERROR );
-	  emit finished(false);
+	  jobFinished(false);
 	  d->running = false;
 	  return;
 	}
@@ -251,13 +277,13 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 
     case K3bDevice::MEDIA_DVD_RAM:
       emit infoMessage( i18n("K3b does not support copying DVD-RAM."), ERROR );
-      emit finished(false);
+      jobFinished(false);
       d->running = false;
       return;
 
     default:
       emit infoMessage( i18n("Unable to determine DVD media type."), ERROR );
-      emit finished(false);
+      jobFinished(false);
       d->running = false;
       return;
     }
@@ -269,10 +295,8 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
       //
       QFileInfo fi( m_imagePath );
       if( !fi.isFile() || 
-	  KMessageBox::warningYesNo( qApp->activeWindow(),
-				     i18n("Do you want to overwrite %1?").arg(m_imagePath),
-				     i18n("File Exists") )
-	  != KMessageBox::Yes) {
+	  questionYesNo( i18n("Do you want to overwrite %1?").arg(m_imagePath),
+			 i18n("File Exists") ) ) {
 	if( fi.isDir() )
 	  m_imagePath = K3b::findTempFile( "iso", m_imagePath );
 	else if( !QFileInfo( m_imagePath.section( '/', 0, -2 ) ).isDir() ) {
@@ -282,7 +306,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 	// else the user specified a file in an existing dir
 
 	emit infoMessage( i18n("Writing image file to %1.").arg(m_imagePath), INFO );
-	emit newSubTask( i18n("Reading source media.") );
+	emit newSubTask( i18n("Reading source medium.") );
       }
 
       //
@@ -293,20 +317,32 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
       QString pathToTest = m_imagePath.left( m_imagePath.findRev( '/' ) );
       if( !K3b::kbFreeOnFs( pathToTest, size, avail ) ) {
 	emit infoMessage( i18n("Unable to determine free space in temporary directory '%1'.").arg(pathToTest), ERROR );
-	emit finished(false);
+	jobFinished(false);
 	d->running = false;
 	return;
       }
       else {
 	if( avail < imageSpaceNeeded/1024 ) {
 	  emit infoMessage( i18n("Not enough space left in temporary directory."), ERROR );
-	  emit finished(false);
+	  jobFinished(false);
 	  d->running = false;
 	  return;
 	}
       }
+
+      d->imageFile.setName( m_imagePath );
+      if( !d->imageFile.open( IO_WriteOnly ) ) {
+	emit infoMessage( i18n("Unable to open '%1' for writing.").arg(m_imagePath), ERROR );
+	jobFinished( false );
+	d->running = false;
+	return;
+      }
     }
 
+    if( K3b::isMounted( m_readerDevice ) ) {
+      emit infoMessage( i18n("Unmounting source medium"), INFO );
+      K3b::unmount( m_readerDevice );
+    }
 
     if( m_onlyCreateImage || !m_onTheFly ) {
       emit newTask( i18n("Creating DVD image") );
@@ -328,7 +364,7 @@ void K3bDvdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
       else {
 	if( d->canceled )
 	  emit canceled();
-	emit finished(false);
+	jobFinished(false);
 	d->running = false;
 	return;
       }
@@ -349,6 +385,9 @@ void K3bDvdCopyJob::cancel()
       d->dataTrackReader->cancel();
     if( d->writerRunning )
       d->writerJob->cancel();
+    d->inPipe.close();
+    d->outPipe.close();
+    d->imageFile.close();
   }
   else {
     kdDebug() << "(K3bDvdCopyJob) not running." << endl;
@@ -358,28 +397,6 @@ void K3bDvdCopyJob::cancel()
 
 void K3bDvdCopyJob::prepareReader()
 {
-//   if( !d->readcdReader ) {
-//     d->readcdReader = new K3bReadcdReader( this );
-//     connect( d->readcdReader, SIGNAL(percent(int)), this, SLOT(slotReaderProgress(int)) );
-//     connect( d->readcdReader, SIGNAL(processedSize(int, int)), this, SLOT(slotReaderProcessedSize(int, int)) );
-//     connect( d->readcdReader, SIGNAL(finished(bool)), this, SLOT(slotReaderFinished(bool)) );
-//     connect( d->readcdReader, SIGNAL(infoMessage(const QString&, int)), this, SIGNAL(infoMessage(const QString&, int)) );
-//     connect( d->readcdReader, SIGNAL(newTask(const QString&)), this, SIGNAL(newSubTask(const QString&)) );
-//     connect( d->readcdReader, SIGNAL(debuggingOutput(const QString&, const QString&)), 
-//              this, SIGNAL(debuggingOutput(const QString&, const QString&)) );
-//   }
-
-//   d->readcdReader->setReadDevice( m_readerDevice );
-//   d->readcdReader->setReadSpeed( 0 ); // MAX
-//   d->readcdReader->setSectorRange( 0, d->lastSector );
-
-//   if( m_onTheFly && !m_onlyCreateImage )
-//     d->readcdReader->writeToFd( d->writerJob->fd() );
-//   else {
-//     d->readcdReader->writeToFd( -1 );
-//     d->readcdReader->setImagePath( m_imagePath );
-//   }
-
   if( !d->dataTrackReader ) {
     d->dataTrackReader = new K3bDataTrackReader( this );
     connect( d->dataTrackReader, SIGNAL(percent(int)), this, SLOT(slotReaderProgress(int)) );
@@ -395,12 +412,14 @@ void K3bDvdCopyJob::prepareReader()
   d->dataTrackReader->setIgnoreErrors( m_ignoreReadErrors );
   d->dataTrackReader->setRetries( m_readRetries );
   d->dataTrackReader->setSectorRange( 0, d->lastSector );
+
   if( m_onTheFly && !m_onlyCreateImage )
-    d->dataTrackReader->writeToFd( d->writerJob->fd() );
-  else {
-    d->dataTrackReader->writeToFd( -1 );
-    d->dataTrackReader->setImagePath( m_imagePath );
-  }
+    d->inPipe.writeToFd( d->writerJob->fd(), true );
+  else 
+    d->inPipe.writeToIODevice( &d->imageFile );
+
+  d->inPipe.open( true );
+  d->dataTrackReader->writeToFd( d->inPipe.in() );
 }
 
 
@@ -430,7 +449,12 @@ void K3bDvdCopyJob::prepareWriter()
   d->writerJob->setWritingMode( d->usedWritingMode );
   d->writerJob->setCloseDvd( true );
  
-  if( d->sourceDiskInfo.numLayers() > 1 ) {
+  //
+  // In case the first layer size is not known let the
+  // split be determined by growisofs
+  //
+  if( d->sourceDiskInfo.numLayers() > 1 &&
+      d->sourceDiskInfo.firstLayerSize() > 0 ) {
     d->writerJob->setLayerBreak( d->sourceDiskInfo.firstLayerSize().lba() );
   }
   else {
@@ -438,22 +462,18 @@ void K3bDvdCopyJob::prepareWriter()
     d->writerJob->setTrackSize( d->lastSector.lba()+1 );
   }
  
-  if( m_onTheFly )
-    d->writerJob->setImageToWrite( QString::null ); // write to stdin
-  else
-    d->writerJob->setImageToWrite( m_imagePath );
+  d->writerJob->setImageToWrite( QString::null ); // write to stdin
 }
 
 
 void K3bDvdCopyJob::slotReaderProgress( int p )
 {
-  if( !m_onTheFly || m_onlyCreateImage )
+  if( !m_onTheFly || m_onlyCreateImage ) {
     emit subPercent( p );
 
-  if( m_onlyCreateImage )
-    emit percent( p );
-  else if( !m_onTheFly )
-    emit percent( p/2 );
+    int bigParts = ( m_onlyCreateImage ? 1 : (m_simulate ? 2 : ( d->verifyData ? m_copies*2 : m_copies ) + 1 ) );
+    emit percent( p/bigParts );
+  }
 }
 
 
@@ -469,12 +489,19 @@ void K3bDvdCopyJob::slotReaderProcessedSize( int p, int c )
 
 void K3bDvdCopyJob::slotWriterProgress( int p )
 {
-  emit subPercent( p );
+  int bigParts = ( m_simulate ? 1 : ( d->verifyData ? m_copies*2 : m_copies ) ) + ( m_onTheFly ? 0 : 1 );
+  int doneParts = ( m_simulate ? 0 : ( d->verifyData ? d->doneCopies*2 : d->doneCopies ) ) + ( m_onTheFly ? 0 : 1 );
+  emit percent( 100*doneParts/bigParts + p/bigParts );
 
-  if( m_onTheFly )
-    emit percent( p );
-  else
-    emit percent( 50 + p/2 );
+  emit subPercent( p );
+}
+
+
+void K3bDvdCopyJob::slotVerificationProgress( int p )
+{
+  int bigParts = ( m_simulate ? 1 : ( d->verifyData ? m_copies*2 : m_copies ) ) + ( m_onTheFly ? 0 : 1 );
+  int doneParts = ( m_simulate ? 0 : ( d->verifyData ? d->doneCopies*2 : d->doneCopies ) ) + ( m_onTheFly ? 0 : 1 ) + 1;
+  emit percent( 100*doneParts/bigParts + p/bigParts );
 }
 
 
@@ -482,6 +509,8 @@ void K3bDvdCopyJob::slotReaderFinished( bool success )
 {
   d->readerRunning = false;
 
+  d->inPipe.close();
+
   // close the socket
   // otherwise growisofs will never quit.
   // FIXME: is it posiible to do this in a generic manner?
@@ -495,29 +524,25 @@ void K3bDvdCopyJob::slotReaderFinished( bool success )
   if( d->canceled ) {
     removeImageFiles();
     emit canceled();
-    emit finished(false);
+    jobFinished(false);
     d->running = false;
   }
 
   if( success ) {
     emit infoMessage( i18n("Successfully read source DVD."), SUCCESS );
     if( m_onlyCreateImage ) {
-      emit finished(true);
+      jobFinished(true);
       d->running = false;
     }
     else {
-      if( m_writerDevice == m_readerDevice )
+      if( m_writerDevice == m_readerDevice ) {
 	// eject the media (we do this blocking to know if it worked
-	// becasue if it did not it might happen that k3b overwrites a CD-RW
+	// because if it did not it might happen that k3b overwrites a CD-RW
 	// source)
 	if( !m_readerDevice->eject() ) {
-	  emit infoMessage( i18n("Unable to eject media."), ERROR );
-	  if( m_removeImageFiles )
-	    removeImageFiles();
-	  d->running = false;
-	  emit finished(false);
-	  return;
+	  blockingInformation( i18n("K3b was unable to eject the source disk. Please do so manually.") );
 	}
+      }
 
       if( !m_onTheFly ) {
 	if( waitForDvd() ) {
@@ -531,21 +556,23 @@ void K3bDvdCopyJob::slotReaderFinished( bool success )
 	  
 	  d->writerRunning = true;
 	  d->writerJob->start();
+	  d->outPipe.writeToFd( d->writerJob->fd(), true );
+	  d->outPipe.open( true );
 	}
 	else {
 	  if( m_removeImageFiles )
 	    removeImageFiles();
 	  if( d->canceled )
 	    emit canceled();
+	  jobFinished(false);
 	  d->running = false;
-	  emit finished(false);
 	}
       }
     }
   }
   else {
     removeImageFiles();
-    emit finished(false);
+    jobFinished(false);
     d->running = false;
   }
 }
@@ -555,6 +582,8 @@ void K3bDvdCopyJob::slotWriterFinished( bool success )
 {
   d->writerRunning = false;
 
+  d->outPipe.close();
+
   // already finished?
   if( !d->running )
     return;
@@ -563,16 +592,44 @@ void K3bDvdCopyJob::slotWriterFinished( bool success )
     if( m_removeImageFiles )
       removeImageFiles();
     emit canceled();
-    emit finished(false);
+    jobFinished(false);
     d->running = false;
   }
 
   if( success ) {
-    d->doneCopies++;
+    emit infoMessage( i18n("Successfully written DVD copy %1.").arg(d->doneCopies+1), INFO );
 
-    emit infoMessage( i18n("Successfully written DVD copy %1.").arg(d->doneCopies), INFO );
+    if( d->verifyData && !m_simulate ) {
+      if( !d->verificationJob ) {
+	d->verificationJob = new K3bVerificationJob( this, this );
+	connect( d->verificationJob, SIGNAL(infoMessage(const QString&, int)),
+		 this, SIGNAL(infoMessage(const QString&, int)) );
+	connect( d->verificationJob, SIGNAL(newTask(const QString&)),
+		 this, SIGNAL(newSubTask(const QString&)) );
+	connect( d->verificationJob, SIGNAL(percent(int)),
+		 this, SLOT(slotVerificationProgress(int)) );
+	connect( d->verificationJob, SIGNAL(percent(int)),
+		 this, SIGNAL(subPercent(int)) );
+	connect( d->verificationJob, SIGNAL(finished(bool)),
+		 this, SLOT(slotVerificationFinished(bool)) );
+	connect( d->verificationJob, SIGNAL(debuggingOutput(const QString&, const QString&)), 
+		 this, SIGNAL(debuggingOutput(const QString&, const QString&)) );
+  
+      }
+      d->verificationJob->setDevice( m_writerDevice );
+      d->verificationJob->addTrack( 1, d->inPipe.checksum(), d->lastSector+1 );
 
-    if( d->doneCopies < m_copies ) {
+      if( m_copies > 1 )
+	emit newTask( i18n("Verifying DVD copy %1").arg(d->doneCopies+1) );
+      else
+	emit newTask( i18n("Verifying DVD copy") );
+
+      emit burning( false );
+
+      d->verificationJob->start();
+    }
+
+    else if( ++d->doneCopies < m_copies ) {
 
       if( waitForDvd() ) {
 	prepareWriter();
@@ -586,7 +643,7 @@ void K3bDvdCopyJob::slotWriterFinished( bool success )
       else {
 	if( d->canceled )
 	  emit canceled();
-	emit finished(false);
+	jobFinished(false);
 	d->running = false;
 	return;
       }
@@ -596,31 +653,75 @@ void K3bDvdCopyJob::slotWriterFinished( bool success )
 	d->readerRunning = true;
 	d->dataTrackReader->start();
       }
+      else {
+	d->outPipe.writeToFd( d->writerJob->fd(), true );
+	d->outPipe.open( true );
+      }
     }
     else {
       if( m_removeImageFiles )
 	removeImageFiles();
       d->running = false;
-      emit finished(true);
+      jobFinished(true);
     }
   }
   else {
     if( m_removeImageFiles )
       removeImageFiles();
     d->running = false;
-    emit finished(false);
+    jobFinished(false);
   }
 }
 
 
-// this is basicly the same code as in K3bDvdJob... :(
+void K3bDvdCopyJob::slotVerificationFinished( bool success )
+{
+  // we simply ignore the results from the verification, the verification
+  // job already emits a message
+  if( ++d->doneCopies < m_copies ) {
+
+    if( waitForDvd() ) {
+      prepareWriter();
+      emit newTask( i18n("Writing DVD copy %1").arg(d->doneCopies+1) );
+
+      emit burning(true);
+
+      d->writerRunning = true;
+      d->writerJob->start();
+    }
+    else {
+      if( d->canceled )
+	emit canceled();
+      jobFinished(false);
+      d->running = false;
+      return;
+    }
+      
+    if( m_onTheFly ) {
+      prepareReader();
+      d->readerRunning = true;
+      d->dataTrackReader->start();
+    }
+    else {
+      d->outPipe.writeToFd( d->writerJob->fd(), true );
+      d->outPipe.open( true );
+    }
+  }
+  else {
+    if( m_removeImageFiles )
+      removeImageFiles();
+    d->running = false;
+    jobFinished( success );
+  }
+}
+
+
+// this is basically the same code as in K3bDvdJob... :(
 // perhaps this should be moved to some K3bGrowisofsHandler which also parses the growisofs output?
 bool K3bDvdCopyJob::waitForDvd()
 {
   int mt = 0;
-  if( m_writingMode == K3b::WRITING_MODE_INCR_SEQ || m_writingMode == K3b::DAO )
-    mt = K3bDevice::MEDIA_DVD_RW_SEQ|K3bDevice::MEDIA_DVD_R_SEQ;
-  else if( m_writingMode == K3b::WRITING_MODE_RES_OVWR ) // we treat DVD+R(W) as restricted overwrite media
+  if( m_writingMode == K3b::WRITING_MODE_RES_OVWR ) // we treat DVD+R(W) as restricted overwrite media
     mt = K3bDevice::MEDIA_DVD_RW_OVWR|K3bDevice::MEDIA_DVD_PLUS_RW|K3bDevice::MEDIA_DVD_PLUS_R;
   else
     mt = K3bDevice::MEDIA_WRITABLE_DVD_SL;
@@ -629,7 +730,7 @@ bool K3bDvdCopyJob::waitForDvd()
   // in case the source is a double layer DVD we made sure above that the writer
   // is capable of writing DVD+R-DL or DVD-R DL and here we wait for a DL DVD
   //
-  if( d->sourceDiskInfo.numLayers() > 1  &&
+  if( d->sourceDiskInfo.numLayers() > 1 &&
       d->sourceDiskInfo.size().mode1Bytes() > 4700372992LL ) {
     mt = K3bDevice::MEDIA_WRITABLE_DVD_DL;
   }
@@ -654,16 +755,16 @@ bool K3bDvdCopyJob::waitForDvd()
       d->usedWritingMode = K3b::WRITING_MODE_RES_OVWR;
 
       if( m_simulate ) {
-	if( KMessageBox::warningYesNo( qApp->activeWindow(),
-				       i18n("K3b does not support simulation with DVD+R(W) media. "
-					    "Do you really want to continue? The media will actually be "
-					    "written to."),
-				       i18n("No Simulation with DVD+R(W)") ) == KMessageBox::No ) {
+	if( !questionYesNo( i18n("K3b does not support simulation with DVD+R(W) media. "
+				 "Do you really want to continue? The media will actually be "
+				 "written to."),
+			    i18n("No Simulation with DVD+R(W)") ) ) {
 	  cancel();
 	  return false;
 	}
 
 //	m_simulate = false;
+	emit newTask( i18n("Writing DVD copy") );
       }
       
       if( m_writingMode != K3b::WRITING_MODE_AUTO && m_writingMode != K3b::WRITING_MODE_RES_OVWR )
@@ -682,13 +783,12 @@ bool K3bDvdCopyJob::waitForDvd()
     // -------------------------------
     else {
       if( m_simulate && !m_writerDevice->dvdMinusTestwrite() ) {
-	if( KMessageBox::warningYesNo( qApp->activeWindow(),
-				       i18n("Your writer (%1 %2) does not support simulation with DVD-R(W) media. "
-					    "Do you really want to continue? The media will be written "
-					    "for real.")
-				       .arg(m_writerDevice->vendor())
-				       .arg(m_writerDevice->description()),
-				       i18n("No Simulation with DVD-R(W)") ) == KMessageBox::No ) {
+	if( !questionYesNo( i18n("Your writer (%1 %2) does not support simulation with DVD-R(W) media. "
+				 "Do you really want to continue? The media will be written "
+				 "for real.")
+			   .arg(m_writerDevice->vendor())
+			   .arg(m_writerDevice->description()),
+			   i18n("No Simulation with DVD-R(W)") ) ) {
 	  cancel();
 	  return false;
 	}
@@ -702,9 +802,9 @@ bool K3bDvdCopyJob::waitForDvd()
       // if this size is wrong
       // With growisofs 5.15 we have the option to specify the size of the image to be written in DAO mode.
       //
-      bool sizeWithDao = ( k3bcore->externalBinManager()->binObject( "growisofs" ) && 
-			   k3bcore->externalBinManager()->binObject( "growisofs" )->version >= K3bVersion( 5, 15, -1 ) );
-
+//       bool sizeWithDao = ( k3bcore->externalBinManager()->binObject( "growisofs" ) && 
+// 			   k3bcore->externalBinManager()->binObject( "growisofs" )->version >= K3bVersion( 5, 15, -1 ) );
+      
 
       // TODO: check for feature 0x21
 
@@ -754,7 +854,7 @@ bool K3bDvdCopyJob::waitForDvd()
 void K3bDvdCopyJob::removeImageFiles()
 {
   if( QFile::exists( m_imagePath ) ) {
-    QFile::remove( m_imagePath );
+    d->imageFile.remove();
     emit infoMessage( i18n("Removed image file %1").arg(m_imagePath), K3bJob::SUCCESS );
   }
 }
@@ -765,12 +865,6 @@ QString K3bDvdCopyJob::jobDescription() const
   if( m_onlyCreateImage ) {
     return i18n("Creating DVD Image");
   }
-  else if( m_simulate ) {
-    if( m_onTheFly )
-      return i18n("Simulating DVD Copy On-The-Fly");
-    else
-      return i18n("Simulating DVD Copy");
-  }
   else {
     if( m_onTheFly )
       return i18n("Copying DVD On-The-Fly");
@@ -787,4 +881,10 @@ QString K3bDvdCopyJob::jobDetails() const
 	      (m_simulate||m_onlyCreateImage) ? 1 : m_copies );
 }
 
+
+void K3bDvdCopyJob::setVerifyData( bool b )
+{
+  d->verifyData = b;
+}
+
 #include "k3bdvdcopyjob.moc"
