@@ -1,10 +1,10 @@
-/* 
+/*
  *
- * $Id: k3bdevice_mmc.cpp 553928 2006-06-22 14:30:38Z trueg $
- * Copyright (C) 2004 Sebastian Trueg <trueg@k3b.org>
+ * $Id: k3bdevice_mmc.cpp 643215 2007-03-16 15:26:01Z trueg $
+ * Copyright (C) 2003-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@ to make the code more readable.
 #include "k3bdevice.h"
 #include "k3bscsicommand.h"
 #include "k3bdeviceglobals.h"
+#include "k3bdebug.h"
 
 #include <string.h>
 
@@ -29,13 +30,14 @@ to make the code more readable.
 bool K3bDevice::Device::testUnitReady() const
 {
   ScsiCommand cmd( this );
+  cmd.enableErrorMessages( false );
   cmd[0] = MMC_TEST_UNIT_READY;
   cmd[5] = 0; // Necessary to set the proper command length
   return( cmd.transport() == 0 );
 }
 
 
-bool K3bDevice::Device::getFeature( unsigned char** data, int& dataLen, unsigned int feature ) const
+bool K3bDevice::Device::getFeature( unsigned char** data, unsigned int& dataLen, unsigned int feature ) const
 {
   unsigned char header[2048];
   ::memset( header, 0, 2048 );
@@ -47,39 +49,39 @@ bool K3bDevice::Device::getFeature( unsigned char** data, int& dataLen, unsigned
   cmd[3] = feature;
   cmd[8] = 8;      // we only read the data length first
   cmd[9] = 0;      // Necessary to set the proper command length
-  if( cmd.transport( TR_DIR_READ, header, 8 ) ) {
-    // again with real length
+
+  // we only read the data length first
+  dataLen = 8;
+  if( cmd.transport( TR_DIR_READ, header, 8 ) )
     dataLen = from4Byte( header ) + 4;
-
-    //
-    // Some buggy firmwares do not return the size of the available data
-    // but the returned data. So we use a high power of 2 to be on the safe side
-    // with these buggy drives.
-    // We cannot use this as default since many firmwares fail with a too high data length.
-    //
-    if( dataLen == 8 ) {
-      cmd[7] = 2048>>8;
-      cmd[8] = 2048;
-      if( cmd.transport( TR_DIR_READ, header, 2048 ) == 0 )
-	dataLen = from2Byte( header ) + 4;
-    }
-
-    *data = new unsigned char[dataLen];
-    ::memset( *data, 0, dataLen );
-
-    cmd[7] = dataLen>>8;
-    cmd[8] = dataLen;
-    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
-      return true;
-    }
-    else {
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": GET CONFIGURATION with real length "
-		<< dataLen << " failed." << endl;
-      delete [] *data;
-    }
-  }
   else
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": GET CONFIGURATION length det failed." << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": GET CONFIGURATION length det failed." << endl;
+
+  //
+  // Some buggy firmwares do not return the size of the available data
+  // but the returned data or something invalid altogether.
+  // So we simply use the maximum possible value to be on the safe side
+  // with these buggy drives.
+  // We cannot use this as default since many firmwares fail with a too high data length.
+  //
+  if( (dataLen-8) % 8 || dataLen <= 8 )
+    dataLen = 0xFFFF;
+
+  // again with real length
+  *data = new unsigned char[dataLen];
+  ::memset( *data, 0, dataLen );
+
+  cmd[7] = dataLen>>8;
+  cmd[8] = dataLen;
+  if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
+    dataLen = QMIN( dataLen, from4Byte( *data ) + 4 );
+    return true;
+  }
+  else {
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": GET CONFIGURATION with real length "
+	      << dataLen << " failed." << endl;
+    delete [] *data;
+  }
 
   return false;
 }
@@ -88,7 +90,7 @@ bool K3bDevice::Device::getFeature( unsigned char** data, int& dataLen, unsigned
 int K3bDevice::Device::featureCurrent( unsigned int feature ) const
 {
   unsigned char* data = 0;
-  int dataLen = 0;
+  unsigned int dataLen = 0;
   if( getFeature( &data, dataLen, feature ) ) {
     int ret = -1;
     if( dataLen >= 11 )
@@ -106,7 +108,7 @@ int K3bDevice::Device::featureCurrent( unsigned int feature ) const
 bool K3bDevice::Device::readIsrc( unsigned int track, QCString& isrc ) const
 {
   unsigned char* data = 0;
-  int dataLen = 0;
+  unsigned int dataLen = 0;
 
   if( readSubChannel( &data, dataLen, 0x3, track ) ) {
     bool isrcValid = false;
@@ -134,7 +136,7 @@ bool K3bDevice::Device::readIsrc( unsigned int track, QCString& isrc ) const
 bool K3bDevice::Device::readMcn( QCString& mcn ) const
 {
   unsigned char* data = 0;
-  int dataLen = 0;
+  unsigned int dataLen = 0;
 
   if( readSubChannel( &data, dataLen, 0x2, 0 ) ) {
     bool mcnValid = false;
@@ -155,13 +157,36 @@ bool K3bDevice::Device::readMcn( QCString& mcn ) const
 }
 
 
-bool K3bDevice::Device::getPerformance( unsigned char** data, int& dataLen,
+bool K3bDevice::Device::getPerformance( unsigned char** data, unsigned int& dataLen,
 					unsigned int type,
 					unsigned int dataType,
 					unsigned int lba ) const
 {
-  unsigned char header[2048];
-  ::memset( header, 0, 2048 );
+  unsigned int descLen = 0;
+  switch( type ) {
+  case 0x0:
+    descLen = 16;
+    break;
+  case 0x1:
+    descLen = 8;
+    break;
+  case 0x2:
+    descLen = 2048;
+    break;
+  case 0x3:
+    descLen = 16;
+    break;
+  case 0x4:
+    descLen = 8;
+    break;
+  case 0x5:
+    descLen = 8; // FIXME: ??
+    break;
+  }
+
+  unsigned char header[8];
+  ::memset( header, 0, 8 );
+  dataLen = 8;
 
   ScsiCommand cmd( this );
   cmd[0] = MMC_GET_PERFORMANCE;
@@ -170,48 +195,54 @@ bool K3bDevice::Device::getPerformance( unsigned char** data, int& dataLen,
   cmd[3] = lba >> 16;
   cmd[4] = lba >> 8;
   cmd[5] = lba;
-  cmd[9] = 1;      // first we read only the header and one descriptor
+  cmd[9] = 0;      // first we read only the header
   cmd[10] = type;
   cmd[11] = 0;     // Necessary to set the proper command length
-  if( cmd.transport( TR_DIR_READ, header, 8 + 16 ) == 0 ) {
+  if( cmd.transport( TR_DIR_READ, header, 8 ) == 0 ) {
     // again with real length
     dataLen = from4Byte( header ) + 4;
 
     // At least one Panasonic drive returns gigantic changing numbers for the data length
-    // which makes K3b crash below when *data cannot be allocated. That's why we cut the 
+    // which makes K3b crash below when *data cannot be allocated. That's why we cut the
     // length here.
-
-    // FIXME: 2048 is a proper upper boundry for the write speed but not for all
+    // FIXME: 2048 is a proper upper boundary for the write speed but not for all
     //        return types. "Defect Status Data" for example might return way more data.
+    // FIXME: I added a "return false" here since there was a situation in which this method
+    //        returned a gigantic dataLen even with this cut. So there is still some bug here.
+    //        Since we only use getPerformance for writing speeds and without a proper length
+    //        those do not make sense it is better to fail here anyway.
     if( dataLen > 2048 ) {
-      kdError() << "(K3bDevice::Device) " << blockDeviceName()
-		<< ": GET PERFORMANCE reports bogus dataLen: " << dataLen
-		<< " cutting to 2048." << endl;
+      k3bDebug() << "(K3bDevice::Device) " << blockDeviceName()
+		 << ": GET PERFORMANCE reports bogus dataLen: " << dataLen << endl;
+      return false;
       dataLen = 2048;
     }
-
-    *data = new unsigned char[dataLen];
-    ::memset( *data, 0, dataLen );
-
-    int numDesc = (dataLen-8)/16;
-
-    cmd[8] = numDesc>>8;
-    cmd[9] = numDesc;
-    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
-      return true;
-    }
-    else {
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName()
-		<< ": GET PERFORMANCE with real length "
-		<< dataLen << " failed." << endl;
-      delete [] *data;
-    }
   }
   else
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName()
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName()
 	      << ": GET PERFORMANCE length det failed." << endl;
 
-  return false;
+  if( (dataLen-8) % descLen || dataLen <= 8 )
+    dataLen = 2048;
+
+  *data = new unsigned char[dataLen];
+  ::memset( *data, 0, dataLen );
+
+  unsigned int numDesc = (dataLen-8)/descLen;
+
+  cmd[8] = numDesc>>8;
+  cmd[9] = numDesc;
+  if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
+    dataLen = QMIN( dataLen, from4Byte( *data ) + 4 );
+    return true;
+  }
+  else {
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName()
+	      << ": GET PERFORMANCE with real length "
+	      << dataLen << " failed." << endl;
+    delete [] *data;
+    return false;
+  }
 }
 
 
@@ -244,7 +275,7 @@ bool K3bDevice::Device::seek( unsigned long lba ) const
 }
 
 
-bool K3bDevice::Device::readTrackInformation( unsigned char** data, int& dataLen, int type, unsigned long value ) const
+bool K3bDevice::Device::readTrackInformation( unsigned char** data, unsigned int& dataLen, int type, int value ) const
 {
   unsigned char header[2048];
   ::memset( header, 0, 2048 );
@@ -264,44 +295,51 @@ bool K3bDevice::Device::readTrackInformation( unsigned char** data, int& dataLen
     cmd[5] = value;
     break;
   default:
-    kdDebug() << "(K3bDevice::readTrackInformation) wrong type parameter: " << type << endl;
+    k3bDebug() << "(K3bDevice::readTrackInformation) wrong type parameter: " << type << endl;
     return false;
   }
 
-  cmd[8] = 4;      // first we read the header
-  if( cmd.transport( TR_DIR_READ, header, 4 ) == 0 ) {
-    // again with real length
+  // first we read the header
+  dataLen = 4;
+  cmd[8] = 4;
+  if( cmd.transport( TR_DIR_READ, header, 4 ) == 0 )
     dataLen = from2Byte( header ) + 2;
-
-    //
-    // Some buggy firmwares do not return the size of the available data
-    // but the returned data. So we use a high power of 2 to be on the safe side
-    // with these buggy drives.
-    // We cannot use this as default since many firmwares fail with a too high data length.
-    //
-    if( dataLen == 4 ) {
-      cmd[7] = 2048>>8;
-      cmd[8] = 2048;
-      if( cmd.transport( TR_DIR_READ, header, 2048 ) == 0 )
-	dataLen = from2Byte( header ) + 2;
-    }
-
-    *data = new unsigned char[dataLen];
-    ::memset( *data, 0, dataLen );
-
-    cmd[7] = dataLen>>8;
-    cmd[8] = dataLen;
-    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
-      return true;
-    }
-    else {
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TRACK INFORMATION with real length "
-		<< dataLen << " failed." << endl;
-      delete [] *data;
-    }
-  }
   else
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TRACK INFORMATION length det failed." << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TRACK INFORMATION length det failed." << endl;
+
+  //
+  // Some buggy firmwares do not return the size of the available data
+  // but the returned data.
+  // So we try to determine the correct size based on the medium type
+  // DVD+R:  40 (MMC4)
+  // DVD-DL: 48 (MMC5)
+  // CD:     36 (MMC2)
+  //
+  if( dataLen <= 4 ) {
+    int m = mediaType();
+    if( m & (MEDIA_DVD_R_DL|MEDIA_DVD_R_DL_SEQ|MEDIA_DVD_R_DL_JUMP) )
+      dataLen = 48;
+    else if( m & (MEDIA_DVD_PLUS_R|MEDIA_DVD_PLUS_R_DL) )
+      dataLen = 40;
+    else
+      dataLen = 36;
+  }
+
+  // again with real length
+  *data = new unsigned char[dataLen];
+  ::memset( *data, 0, dataLen );
+
+  cmd[7] = dataLen>>8;
+  cmd[8] = dataLen;
+  if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
+    dataLen = QMIN( dataLen, from2Byte( *data ) + 2u );
+    return true;
+  }
+  else {
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TRACK INFORMATION with real length "
+	      << dataLen << " failed." << endl;
+    delete [] *data;
+  }
 
   return false;
 }
@@ -309,7 +347,7 @@ bool K3bDevice::Device::readTrackInformation( unsigned char** data, int& dataLen
 
 
 bool K3bDevice::Device::read10( unsigned char* data,
-				int dataLen,
+				unsigned int dataLen,
 				unsigned long startAdress,
 				unsigned int length,
 				bool fua ) const
@@ -328,7 +366,7 @@ bool K3bDevice::Device::read10( unsigned char* data,
   cmd[9] = 0;      // Necessary to set the proper command length
 
   if( cmd.transport( TR_DIR_READ, data, dataLen ) ) {
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ 10 failed!" << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ 10 failed!" << endl;
     return false;
   }
   else
@@ -337,7 +375,7 @@ bool K3bDevice::Device::read10( unsigned char* data,
 
 
 bool K3bDevice::Device::read12( unsigned char* data,
-				int dataLen,
+				unsigned int dataLen,
 				unsigned long startAdress,
 				unsigned long length,
 				bool streaming,
@@ -360,7 +398,7 @@ bool K3bDevice::Device::read12( unsigned char* data,
   cmd[11] = 0;      // Necessary to set the proper command length
 
   if( cmd.transport( TR_DIR_READ, data, dataLen ) ) {
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ 12 failed!" << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ 12 failed!" << endl;
     return false;
   }
   else
@@ -369,7 +407,7 @@ bool K3bDevice::Device::read12( unsigned char* data,
 
 
 bool K3bDevice::Device::readCd( unsigned char* data,
-				int dataLen,
+				unsigned int dataLen,
 				int sectorType,
 				bool dap,
 				unsigned long startAdress,
@@ -404,7 +442,7 @@ bool K3bDevice::Device::readCd( unsigned char* data,
   cmd[11] = 0;      // Necessary to set the proper command length
 
   if( cmd.transport( TR_DIR_READ, data, dataLen ) ) {
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ CD failed!" << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ CD failed!" << endl;
     return false;
   }
   else {
@@ -414,7 +452,7 @@ bool K3bDevice::Device::readCd( unsigned char* data,
 
 
 bool K3bDevice::Device::readCdMsf( unsigned char* data,
-				   int dataLen,
+				   unsigned int dataLen,
 				   int sectorType,
 				   bool dap,
 				   const K3b::Msf& startAdress,
@@ -448,7 +486,7 @@ bool K3bDevice::Device::readCdMsf( unsigned char* data,
   cmd[11] = 0;      // Necessary to set the proper command length
 
   if( cmd.transport( TR_DIR_READ, data, dataLen ) ) {
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ CD MSF failed!" << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ CD MSF failed!" << endl;
     return false;
   }
   else
@@ -456,7 +494,7 @@ bool K3bDevice::Device::readCdMsf( unsigned char* data,
 }
 
 
-bool K3bDevice::Device::readSubChannel( unsigned char** data, int& dataLen,
+bool K3bDevice::Device::readSubChannel( unsigned char** data, unsigned int& dataLen,
 					unsigned int subchannelParam,
 					unsigned int trackNumber ) const
 {
@@ -468,48 +506,70 @@ bool K3bDevice::Device::readSubChannel( unsigned char** data, int& dataLen,
   cmd[2] = 0x40;    // SUBQ
   cmd[3] = subchannelParam;
   cmd[6] = trackNumber;   // only used when subchannelParam == 03h (ISRC)
-  cmd[8] = 4;      // first we read the header
+  cmd[8] = 4;
   cmd[9] = 0;      // Necessary to set the proper command length
-  if( cmd.transport( TR_DIR_READ, header, 4 ) == 0 ) {
-    // again with real length
+
+  // first we read the header
+  dataLen = 4;
+  if( cmd.transport( TR_DIR_READ, header, 4 ) == 0 )
     dataLen = from2Byte( &header[2] ) + 4;
-
-    //
-    // Some buggy firmwares do not return the size of the available data
-    // but the returned data. So we use a high power of 2 to be on the safe side
-    // with these buggy drives.
-    // We cannot use this as default since many firmwares fail with a too high data length.
-    //
-    if( dataLen == 4 ) {
-      cmd[7] = 2048>>8;
-      cmd[8] = 2048;
-      if( cmd.transport( TR_DIR_READ, header, 2048 ) == 0 )
-	dataLen = from2Byte( &header[2] ) + 4;
-    }
-
-    *data = new unsigned char[dataLen];
-    ::memset( *data, 0, dataLen );
-
-    cmd[7] = dataLen>>8;
-    cmd[8] = dataLen;
-    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
-      return true;
-    }
-    else {
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ SUB-CHANNEL with real length "
-		<< dataLen << " failed." << endl;
-      delete [] *data;
-    }
-  }
   else
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ SUB-CHANNEL length det failed." << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ SUB-CHANNEL length det failed." << endl;
+
+  //
+  // Some buggy firmwares do not return the size of the available data
+  // but the returned data. So we simply use the maximum possible value to be on the safe side
+  // with these buggy drives.
+  // We cannot use this as default since many firmwares fail with a too high data length.
+  //
+  if( dataLen <= 4 )
+    dataLen = 0xFFFF;
+
+  // again with real length
+  *data = new unsigned char[dataLen];
+  ::memset( *data, 0, dataLen );
+
+  cmd[7] = dataLen>>8;
+  cmd[8] = dataLen;
+  if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
+    dataLen = QMIN( dataLen, from2Byte( (*data)+2 ) + 4u );
+    return true;
+  }
+  else {
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ SUB-CHANNEL with real length "
+	      << dataLen << " failed." << endl;
+    delete [] *data;
+  }
 
   return false;
 }
 
 
-bool K3bDevice::Device::readTocPmaAtip( unsigned char** data, int& dataLen, int format, bool time, int track ) const
+bool K3bDevice::Device::readTocPmaAtip( unsigned char** data, unsigned int& dataLen, int format, bool time, int track ) const
 {
+  unsigned int descLen = 0;
+
+  switch( format ) {
+  case 0x0:
+    descLen = 8;
+    break;
+  case 0x1:
+    descLen = 8;
+    break;
+  case 0x2:
+    descLen = 11;
+    break;
+  case 0x3:
+    descLen = 11;
+    break;
+  case 0x4:
+    descLen = 4; // MMC2: 24 and MMC4: 28, so we use the highest common factor
+    break;
+  case 0x5:
+    descLen = 18;
+    break;
+  }
+
   unsigned char header[2048];
   ::memset( header, 0, 2048 );
 
@@ -518,147 +578,160 @@ bool K3bDevice::Device::readTocPmaAtip( unsigned char** data, int& dataLen, int
   cmd[1] = ( time ? 0x2 : 0x0 );
   cmd[2] = format & 0x0F;
   cmd[6] = track;
-  cmd[8] = 2;      // we only read the length first
+  cmd[8] = 4;
   cmd[9] = 0;      // Necessary to set the proper command length
 
-  if( cmd.transport( TR_DIR_READ, header, 2 ) == 0 ) {
-    // again with real length
+  // we only read the header
+  dataLen = 4;
+  if( cmd.transport( TR_DIR_READ, header, 4 ) == 0 )
     dataLen = from2Byte( header ) + 2;
-
-    //
-    // Some buggy firmwares do not return the size of the available data
-    // but the returned data. So we use a high power of 2 to be on the safe side
-    // with these buggy drives.
-    // We cannot use this as default since many firmwares fail with a too high data length.
-    //
-    if( dataLen == 2 ) {
-      cmd[7] = 2048>>8;
-      cmd[8] = 2048;
-      if( cmd.transport( TR_DIR_READ, header, 2048 ) == 0 )
-	dataLen = from2Byte( header ) + 2;
-    }
-
-    *data = new unsigned char[dataLen];
-    ::memset( *data, 0, dataLen );
-
-    cmd[7] = dataLen>>8;
-    cmd[8] = dataLen;
-    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 )
-      return true;
-    else {
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TOC/PMA/ATIP format "
-		<< format << " with real length "
-		<< dataLen << " failed." << endl;
-      delete [] *data;
-    }
-  }
   else
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TOC/PMA/ATIP length det failed." << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TOC/PMA/ATIP length det failed." << endl;
+
+  //
+  // Some buggy firmwares return an invalid size here
+  // So we simply use the maximum possible value to be on the safe side
+  // with these buggy drives.
+  // We cannot use this as default since many firmwares fail with a too high data length.
+  //
+  if( (dataLen-4) % descLen || dataLen < 4+descLen ) {
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TOC/PMA/ATIP invalid length returned: " << dataLen << endl;
+    dataLen = 0xFFFF;
+  }
+
+  //
+  // Not all drives like uneven numbers
+  //
+  if( dataLen%2 )
+    ++dataLen;
+
+  // again with real length
+  *data = new unsigned char[dataLen];
+  ::memset( *data, 0, dataLen );
+
+  cmd[7] = dataLen>>8;
+  cmd[8] = dataLen;
+  if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
+    dataLen = QMIN( dataLen, from2Byte( *data ) + 2u );
+    if( (dataLen-4) % descLen || dataLen < 4+descLen ) {
+        // useless length
+        delete [] *data;
+        return false;
+    }
+    else
+        return true;
+  }
+  else {
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ TOC/PMA/ATIP format "
+	      << format << " with real length "
+	      << dataLen << " failed." << endl;
+    delete [] *data;
+  }
 
   return false;
 }
 
 
-bool K3bDevice::Device::mechanismStatus( unsigned char** data, int& dataLen ) const
+bool K3bDevice::Device::mechanismStatus( unsigned char** data, unsigned int& dataLen ) const
 {
   unsigned char header[2048];
   ::memset( header, 0, 2048 );
 
   ScsiCommand cmd( this );
   cmd[0] = MMC_MECHANISM_STATUS;
-  cmd[9] = 8;     // first we read the header
+  cmd[9] = 8;
   cmd[11] = 0;    // Necessary to set the proper command length
-  if( cmd.transport( TR_DIR_READ, header, 8 ) == 0 ) {
-    // again with real length
+
+  // first we read the header
+  dataLen = 8;
+  if( cmd.transport( TR_DIR_READ, header, 8 ) == 0 )
     dataLen = from4Byte( &header[6] ) + 8;
-
-    //
-    // Some buggy firmwares do not return the size of the available data
-    // but the returned data. So we use a high power of 2 to be on the safe side
-    // with these buggy drives.
-    // We cannot use this as default since many firmwares fail with a too high data length.
-    //
-    if( dataLen == 8 ) {
-      cmd[8] = 2048>>8;
-      cmd[9] = 2048;
-      if( cmd.transport( TR_DIR_READ, header, 2048 ) == 0 )
-	dataLen = from2Byte( &header[6] ) + 8;
-    }
-
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MECHANISM STATUS "
-	      << (int)header[5] << " slots." << endl;
-
-    *data = new unsigned char[dataLen];
-    ::memset( *data, 0, dataLen );
-
-    cmd[8] = dataLen>>8;
-    cmd[9] = dataLen;
-    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
-      return true;
-    }
-    else {
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MECHANISM STATUS with real length "
-		<< dataLen << " failed." << endl;
-      delete [] *data;
-    }
-  }
   else
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MECHANISM STATUS length det failed." << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MECHANISM STATUS length det failed." << endl;
+
+  //
+  // Some buggy firmwares do not return the size of the available data
+  // but the returned data or something invalid altogether.
+  // So we simply use the maximum possible value to be on the safe side
+  // with these buggy drives.
+  // We cannot use this as default since many firmwares fail with a too high data length.
+  //
+  if( (dataLen-8) % 4 || dataLen <= 8 )
+    dataLen = 0xFFFF;
+
+  k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MECHANISM STATUS "
+	    << (int)header[5] << " slots." << endl;
+
+  // again with real length
+  *data = new unsigned char[dataLen];
+  ::memset( *data, 0, dataLen );
+
+  cmd[8] = dataLen>>8;
+  cmd[9] = dataLen;
+  if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
+    dataLen = QMIN( dataLen, from4Byte( (*data)+6 ) + 8 );
+    return true;
+  }
+  else {
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MECHANISM STATUS with real length "
+	      << dataLen << " failed." << endl;
+    delete [] *data;
+  }
 
   return false;
 }
 
 
 
-bool K3bDevice::Device::modeSense( unsigned char** pageData, int& pageLen, int page ) const
+bool K3bDevice::Device::modeSense( unsigned char** pageData, unsigned int& pageLen, int page ) const
 {
   unsigned char header[2048];
   ::memset( header, 0, 2048 );
 
   ScsiCommand cmd( this );
   cmd[0] = MMC_MODE_SENSE;
-  cmd[1] = 0x08;        // Disable Block Descriptors
-  cmd[2] = page;
-  cmd[8] = 8;           // first we determine the data length
+  cmd[1] = 0x8;        // Disable Block Descriptors
+  cmd[2] = page & 0x3F;
+  cmd[8] = 8;
   cmd[9] = 0;           // Necessary to set the proper command length
-  if( cmd.transport( TR_DIR_READ, header, 8 ) == 0 ) {
-    // again with real length
+
+  // first we determine the data length
+  pageLen = 8;
+  if( cmd.transport( TR_DIR_READ, header, 8 ) == 0 )
     pageLen = from2Byte( header ) + 2;
-
-    //
-    // Some buggy firmwares do not return the size of the available data
-    // but the returned data. So we use a high power of 2 to be on the safe side
-    // with these buggy drives.
-    // We cannot use this as default since many firmwares fail with a too high data length.
-    //
-    if( pageLen == 8 ) {
-      cmd[7] = 2048>>8;
-      cmd[8] = 2048;
-      if( cmd.transport( TR_DIR_READ, header, 2048 ) == 0 )
-	pageLen = from2Byte( header ) + 2;
-    }
-
-    *pageData = new unsigned char[pageLen];
-    ::memset( *pageData, 0, pageLen );
-
-    cmd[7] = pageLen>>8;
-    cmd[8] = pageLen;
-    if( cmd.transport( TR_DIR_READ, *pageData, pageLen ) == 0 )
-      return true;
-    else {
-      delete [] *pageData;
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MODE SENSE with real length "
-		<< pageLen << " failed." << endl;
-    }
-  }
   else
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MODE SENSE length det failed." << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MODE SENSE length det failed." << endl;
+
+  //
+  // Some buggy firmwares do not return the size of the available data
+  // but the returned data. So we simply use the maximum possible value to be on the safe side
+  // with these buggy drives.
+  // We cannot use this as default since many firmwares fail with a too high data length.
+  //
+  if( pageLen == 8 )
+    pageLen = 0xFFFF;
+
+  // again with real length
+  *pageData = new unsigned char[pageLen];
+  ::memset( *pageData, 0, pageLen );
+
+  cmd[7] = pageLen>>8;
+  cmd[8] = pageLen;
+  if( cmd.transport( TR_DIR_READ, *pageData, pageLen ) == 0 ) {
+    pageLen = QMIN( pageLen, from2Byte( *pageData ) + 2u );
+    return true;
+  }
+  else {
+    delete [] *pageData;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": MODE SENSE with real length "
+	      << pageLen << " failed." << endl;
+  }
 
   return false;
 }
 
 
-bool K3bDevice::Device::modeSelect( unsigned char* page, int pageLen, bool pf, bool sp ) const
+bool K3bDevice::Device::modeSelect( unsigned char* page, unsigned int pageLen, bool pf, bool sp ) const
 {
   page[0] = 0;
   page[1] = 0;
@@ -717,9 +790,9 @@ bool K3bDevice::Device::readFormatCapacity( int wantedFormat, K3b::Msf& r,
   cmd[9] = 0;      // Necessary to set the proper command length
   if( cmd.transport( TR_DIR_READ, buffer, maxLen ) == 0 ) {
 
-    int realLength = buffer[3] + 4;
+    unsigned int realLength = buffer[3] + 4;
 
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << " READ FORMAT CAPACITY: Current/Max "
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << " READ FORMAT CAPACITY: Current/Max "
 	      << (int)(buffer[8]&0x3) << " " << from4Byte( &buffer[4] ) << endl;
 
     if( currentMax )
@@ -734,9 +807,9 @@ bool K3bDevice::Device::readFormatCapacity( int wantedFormat, K3b::Msf& r,
     // 2 - formatted. Here we get the used capacity (lead-in to last lead-out/border-out)
     // 3 - No media present
     //
-    for( int i = 12; i < realLength-4; i+=8 ) {
+    for( unsigned int i = 12; i < realLength-4; i+=8 ) {
       int format = (int)((buffer[i+4]>>2)&0x3f);
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << " READ FORMAT CAPACITY: "
+      k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << " READ FORMAT CAPACITY: "
 		<< format << " " << from4Byte( &buffer[i] )
 		<< " " << (int)( buffer[i+5] << 16 & 0xFF0000 |
 				 buffer[i+6] << 8  & 0xFF00 |
@@ -754,52 +827,70 @@ bool K3bDevice::Device::readFormatCapacity( int wantedFormat, K3b::Msf& r,
 }
 
 
-bool K3bDevice::Device::readDiscInfo( unsigned char** data, int& dataLen ) const
+bool K3bDevice::Device::readDiscInformation( unsigned char** data, unsigned int& dataLen ) const
 {
   unsigned char header[2];
   ::memset( header, 0, 2 );
 
   ScsiCommand cmd( this );
-  cmd[0] = MMC_READ_DISK_INFORMATION;
+  cmd[0] = MMC_READ_DISC_INFORMATION;
   cmd[8] = 2;
   cmd[9] = 0;      // Necessary to set the proper command length
 
-  if( cmd.transport( TR_DIR_READ, header, 2 ) == 0 ) {
-    // again with real length
+  if( cmd.transport( TR_DIR_READ, header, 2 ) == 0 )
     dataLen = from2Byte( header ) + 2;
+  else
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName()
+	       << ": READ DISC INFORMATION length det failed" << endl;
 
-    *data = new unsigned char[dataLen];
-    ::memset( *data, 0, dataLen );
+  if( dataLen < 32 ) {
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName()
+	       << ": Device reports bogus disc information length of " << dataLen << endl;
+    dataLen = 32;
+  }
 
-    cmd[7] = dataLen>>8;
-    cmd[8] = dataLen;
-    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 )
-      return true;
-    else {
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ DISC INFORMATION with real length "
-		<< dataLen << " failed." << endl;
-      delete [] *data;
-    }
+  *data = new unsigned char[dataLen];
+  ::memset( *data, 0, dataLen );
+
+  cmd[7] = dataLen>>8;
+  cmd[8] = dataLen;
+  if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
+    dataLen = QMIN( dataLen, from2Byte( *data ) + 2u );
+    return true;
   }
   else {
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ DISC INFORMATION length det failed" << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ DISC INFORMATION with real length "
+	      << dataLen << " failed." << endl;
+    delete [] *data;
   }
 
   return false;
 }
 
 
-bool K3bDevice::Device::readDvdStructure( unsigned char** data, int& dataLen, 
+bool K3bDevice::Device::readDvdStructure( unsigned char** data, unsigned int& dataLen,
 					  unsigned int format,
 					  unsigned int layer,
 					  unsigned long adress,
 					  unsigned int agid ) const
+{
+  return readDiscStructure( data, dataLen, 0x0, format, layer, adress, agid );
+}
+
+
+bool K3bDevice::Device::readDiscStructure( unsigned char** data, unsigned int& dataLen,
+					   unsigned int mediaType,
+					   unsigned int format,
+					   unsigned int layer,
+					   unsigned long adress,
+					   unsigned int agid ) const
 {
   unsigned char header[4];
   ::memset( header, 0, 4 );
 
   ScsiCommand cmd( this );
   cmd[0] = MMC_READ_DVD_STRUCTURE;
+  cmd[1] = mediaType & 0xF;
   cmd[2] = adress>>24;
   cmd[3] = adress>>16;
   cmd[4] = adress>>8;
@@ -810,7 +901,7 @@ bool K3bDevice::Device::readDvdStructure( unsigned char** data, int& dataLen,
   cmd[11] = 0;      // Necessary to set the proper command length
 
   cmd[9] = 4;
-  if( cmd.transport( TR_DIR_READ, header, 2 ) == 0 ) {
+  if( cmd.transport( TR_DIR_READ, header, 4 ) == 0 ) {
     // again with real length
     dataLen = from2Byte( header ) + 2;
 
@@ -819,14 +910,17 @@ bool K3bDevice::Device::readDvdStructure( unsigned char** data, int& dataLen,
 
     cmd[8] = dataLen>>8;
     cmd[9] = dataLen;
-    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 )
+    if( cmd.transport( TR_DIR_READ, *data, dataLen ) == 0 ) {
+      dataLen = QMIN( dataLen, from2Byte( *data ) + 2u );
       return true;
+    }
     else {
-      kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ DVD STRUCTURE with real length failed." << endl;
+      k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ DVD STRUCTURE with real length failed." << endl;
+      delete [] *data;
     }
   }
   else
-    kdDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ DVD STRUCTURE length det failed" << endl;
+    k3bDebug() << "(K3bDevice::Device) " << blockDeviceName() << ": READ DVD STRUCTURE length det failed" << endl;
 
   return false;
 }
@@ -845,7 +939,7 @@ int K3bDevice::Device::readBufferCapacity( long long& bufferLength, long long& b
   if( r )
     return r;
 
-  int dataLength = from2Byte( data );
+  unsigned int dataLength = from2Byte( data );
 
   if( dataLength >= 10 ) {
     bufferLength = from4Byte( &data[4] );
