@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3baudioburndialog.cpp 475433 2005-10-29 09:31:42Z trueg $
+ * $Id: k3baudioburndialog.cpp 623768 2007-01-15 13:33:55Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -103,11 +103,10 @@ K3bAudioBurnDialog::K3bAudioBurnDialog(K3bAudioDoc* _doc, QWidget *parent, const
 
   addPage( advancedTab, i18n("Advanced") );
 
-  connect( m_writerSelectionWidget, SIGNAL(writingAppChanged(int)), this, SLOT(toggleAllOptions()) );
-  connect( m_checkNormalize, SIGNAL(toggled(bool)), this, SLOT(toggleAllOptions()) );
-  connect( m_writingModeWidget, SIGNAL(writingModeChanged(int)), this, SLOT(toggleAllOptions()) );
-
-  readSettings();
+  connect( m_writerSelectionWidget, SIGNAL(writingAppChanged(int)), this, SLOT(slotToggleAll()) );
+  connect( m_checkNormalize, SIGNAL(toggled(bool)), this, SLOT(slotNormalizeToggled(bool)) );
+  connect( m_checkCacheImage, SIGNAL(toggled(bool)), this, SLOT(slotCacheImageToggled(bool)) );
+  connect( m_writingModeWidget, SIGNAL(writingModeChanged(int)), this, SLOT(slotToggleAll()) );
 
   // ToolTips
   // -------------------------------------------------------------------------
@@ -175,7 +174,7 @@ void K3bAudioBurnDialog::readSettings()
   if( !doc()->tempDir().isEmpty() )
     m_tempDirSelectionWidget->setTempPath( doc()->tempDir() );
 
-  toggleAllOptions();
+  toggleAll();
 }
 
 
@@ -188,10 +187,10 @@ void K3bAudioBurnDialog::loadK3bDefaults()
   m_checkNormalize->setChecked(false);
 
   m_comboParanoiaMode->setCurrentItem( 0 );
-  m_checkAudioRippingIgnoreReadErrors->setChecked( false );
-  m_spinAudioRippingReadRetries->setValue( 128 );
+  m_checkAudioRippingIgnoreReadErrors->setChecked( true );
+  m_spinAudioRippingReadRetries->setValue( 5 );
 
-  toggleAllOptions();
+  toggleAll();
 }
 
 
@@ -204,10 +203,10 @@ void K3bAudioBurnDialog::loadUserDefaults( KConfigBase* c )
   m_checkNormalize->setChecked( c->readBoolEntry( "normalize", false ) );
 
   m_comboParanoiaMode->setCurrentItem( c->readNumEntry( "paranoia mode", 0 ) );
-  m_checkAudioRippingIgnoreReadErrors->setChecked( c->readBoolEntry( "ignore read errors", false ) );
-  m_spinAudioRippingReadRetries->setValue( c->readNumEntry( "read retries", 128 ) );
+  m_checkAudioRippingIgnoreReadErrors->setChecked( c->readBoolEntry( "ignore read errors", true ) );
+  m_spinAudioRippingReadRetries->setValue( c->readNumEntry( "read retries", 5 ) );
 
-  toggleAllOptions();
+  toggleAll();
 }
 
 
@@ -224,15 +223,14 @@ void K3bAudioBurnDialog::saveUserDefaults( KConfigBase* c )
   c->writeEntry( "read retries", m_spinAudioRippingReadRetries->value() );
 }
 
-void K3bAudioBurnDialog::toggleAllOptions()
+void K3bAudioBurnDialog::toggleAll()
 {
-  K3bProjectBurnDialog::toggleAllOptions();
+  K3bProjectBurnDialog::toggleAll();
 
   bool cdrecordOnTheFly = false;
   bool cdrecordCdText = false;
   if ( k3bcore->externalBinManager()->binObject("cdrecord") ) {
-    cdrecordOnTheFly = k3bcore->externalBinManager()->binObject("cdrecord")->version
-      >= K3bVersion( 2, 1, -1, "a13" );
+    cdrecordOnTheFly = k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "audio-stdin" );
     cdrecordCdText = k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "cdtext" );
   }
 
@@ -252,20 +250,15 @@ void K3bAudioBurnDialog::toggleAllOptions()
     m_cdtextWidget->setEnabled( !m_checkOnlyCreateImage->isChecked() );
   }
 
-  m_checkOnTheFly->setEnabled( !m_checkOnlyCreateImage->isChecked() && 
-			       onTheFly && 
-			       !m_checkNormalize->isChecked() );
-  if( !onTheFly || m_checkNormalize->isChecked() )
-    m_checkOnTheFly->setChecked( false );
+  m_checkCacheImage->setEnabled( !m_checkOnlyCreateImage->isChecked() && 
+				 onTheFly );
+  if( !onTheFly )
+    m_checkCacheImage->setChecked( true );
   m_cdtextWidget->setEnabled( !m_checkOnlyCreateImage->isChecked() &&
 			      cdText && 
 			      m_writingModeWidget->writingMode() != K3b::TAO );
   if( !cdText || m_writingModeWidget->writingMode() == K3b::TAO )
     m_cdtextWidget->setChecked(false);
-
-  // we are not able to normalize in on-the-fly mode
-  m_checkNormalize->setDisabled( !k3bcore->externalBinManager()->foundBin( "normalize" ) ||
-				 ( m_checkOnTheFly->isChecked() && !m_checkOnlyCreateImage->isChecked() ) );
 }
 
 
@@ -299,4 +292,50 @@ void K3bAudioBurnDialog::showEvent( QShowEvent* e )
   K3bProjectBurnDialog::showEvent(e);
 }
 
+
+void K3bAudioBurnDialog::slotNormalizeToggled( bool on )
+{
+  if( on ) {
+    // we are not able to normalize in on-the-fly mode
+    if( !k3bcore->externalBinManager()->foundBin( "normalize-audio" ) ) {
+      KMessageBox::sorry( this, i18n("<p><b>External program <em>normalize-audio</em> is not installed.</b>"
+				     "<p>K3b uses <em>normalize-audio</em> (http://www1.cs.columbia.edu/~cvaill/normalize/) "
+				     "to normalize audio tracks. In order to "
+				     "use this functionality, please install it first.") );
+      m_checkNormalize->setChecked( false );
+    }
+    else if( !m_checkCacheImage->isChecked() && !m_checkOnlyCreateImage->isChecked() ) {
+      if( KMessageBox::warningYesNo( this, i18n("<p>K3b is not able to normalize audio tracks when burning on-the-fly. "
+						"The external program used for this task only supports normalizing a set "
+						"of audio files."),
+				     QString::null,
+				     i18n("Disable normalization"),
+				     i18n("Disable on-the-fly burning"),
+				     "audioProjectNormalizeOrOnTheFly" ) == KMessageBox::Yes )
+	m_checkNormalize->setChecked( false );
+      else
+	m_checkCacheImage->setChecked( true );
+    }
+  }
+}
+
+
+void K3bAudioBurnDialog::slotCacheImageToggled( bool on )
+{
+  if( !on ) {
+    if( m_checkNormalize->isChecked() ) {
+      if( KMessageBox::warningYesNo( this, i18n("<p>K3b is not able to normalize audio tracks when burning on-the-fly. "
+						"The external program used for this task only supports normalizing a set "
+						"of audio files."),
+				     QString::null,
+				     i18n("Disable normalization"),
+				     i18n("Disable on-the-fly burning"),
+				     "audioProjectNormalizeOrOnTheFly" ) == KMessageBox::Yes )
+	m_checkNormalize->setChecked( false );
+      else
+	m_checkCacheImage->setChecked( true );
+    }
+  }
+}
+
 #include "k3baudioburndialog.moc"
