@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3bmixedburndialog.cpp 476670 2005-11-01 19:12:27Z trueg $
+ * $Id: k3bmixedburndialog.cpp 627521 2007-01-26 22:39:53Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
 #include "k3bmixedview.h"
 
 #include <k3bdataimagesettingswidget.h>
-#include <k3bdataadvancedimagesettingswidget.h>
-#include <k3bdatavolumedescwidget.h>
 #include <k3bdatadoc.h>
 #include <k3baudiodoc.h>
 #include <k3bdevice.h>
@@ -36,6 +34,7 @@
 #include <k3bversion.h>
 #include <k3bcore.h>
 #include <k3baudiotrackplayer.h>
+#include <k3bintmapcombobox.h>
 
 #include <qtabwidget.h>
 #include <qcheckbox.h>
@@ -57,6 +56,7 @@
 #include <kconfig.h>
 #include <kapplication.h>
 #include <kdebug.h>
+#include <kmessagebox.h>
 
 
 K3bMixedBurnDialog::K3bMixedBurnDialog( K3bMixedDoc* doc, QWidget *parent, const char *name, bool modal )
@@ -70,37 +70,23 @@ K3bMixedBurnDialog::K3bMixedBurnDialog( K3bMixedDoc* doc, QWidget *parent, const
 
   m_checkOnlyCreateImage->hide();
 
-  setupSettingsPage();
-
   // create cd-text page
   m_cdtextWidget = new K3bAudioCdTextWidget( this );
   addPage( m_cdtextWidget, i18n("CD-Text") );
 
-  // create volume descriptor tab
-  m_volumeDescWidget = new K3bDataVolumeDescWidget( this );
-  m_volumeDescWidget->layout()->setMargin( marginHint() );
-  addPage( m_volumeDescWidget, i18n("Volume Desc") );
-
   // create image settings tab
   m_imageSettingsWidget = new K3bDataImageSettingsWidget( this );
-  m_imageSettingsWidget->layout()->setMargin( marginHint() );
   addPage( m_imageSettingsWidget, i18n("Filesystem") );
 
-  // create advanced image settings tab
-  m_advancedImageSettingsWidget = new K3bDataAdvancedImageSettingsWidget( this );
-  m_advancedImageSettingsWidget->layout()->setMargin( marginHint() );
-  addPage( m_advancedImageSettingsWidget, i18n("Advanced") );
-
-  createContextHelp();
+  setupSettingsPage();
 
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
   m_optionGroupLayout->addItem( spacer );
 
-  connect( m_checkNormalize, SIGNAL(toggled(bool)), this, SLOT(toggleAllOptions()) );
-  connect( m_writerSelectionWidget, SIGNAL(writingAppChanged(int)), this, SLOT(toggleAllOptions()) );
-  connect( m_writingModeWidget, SIGNAL(writingModeChanged(int)), this, SLOT(toggleAllOptions()) );
-
-  readSettings();
+  connect( m_checkNormalize, SIGNAL(toggled(bool)), this, SLOT(slotNormalizeToggled(bool)) );
+  connect( m_checkCacheImage, SIGNAL(toggled(bool)), this, SLOT(slotCacheImageToggled(bool)) );
+  connect( m_writerSelectionWidget, SIGNAL(writingAppChanged(int)), this, SLOT(slotToggleAll()) );
+  connect( m_writingModeWidget, SIGNAL(writingModeChanged(int)), this, SLOT(slotToggleAll()) );
 }
 
 
@@ -111,74 +97,52 @@ void K3bMixedBurnDialog::setupSettingsPage()
   QGroupBox* groupDataMode = new QGroupBox( 1, Qt::Vertical, i18n("Datatrack Mode"), w );
   m_dataModeWidget = new K3bDataModeWidget( groupDataMode );
 
-  QGroupBox* groupNormalize = new QGroupBox( 1, Qt::Vertical, i18n("Misc."), w );
+  QGroupBox* groupNormalize = new QGroupBox( 1, Qt::Vertical, i18n("Misc"), w );
   m_checkNormalize = K3bStdGuiItems::normalizeCheckBox( groupNormalize );
 
-  m_groupMixedType = new QButtonGroup( 4, Qt::Vertical, i18n("Mixed Mode Type"), w );
-
-  // Enhanced music CD/CD Extra/CD Plus format (Blue Book)
-  // to fulfill the standard we also need the special file structure
-  // but in the case of our simple mixed mode cd we allow to create blue book cds without
-  // these special files and directories
-  m_radioMixedTypeSessions = new QRadioButton( i18n("Data in second session (CD-Extra)"), m_groupMixedType );
-
-  // standard mixed mode
-  m_radioMixedTypeFirstTrack = new QRadioButton( i18n("Data in first track"), m_groupMixedType );
-
-  // is this a standard?
-  m_radioMixedTypeLastTrack = new QRadioButton( i18n("Data in last track"), m_groupMixedType );
-
-  m_groupMixedType->setExclusive(true);
+  QGroupBox* groupMixedType = new QGroupBox( 1, Qt::Vertical, i18n("Mixed Mode Type"), w );
+  m_comboMixedModeType = new K3bIntMapComboBox( groupMixedType );
 
+  m_comboMixedModeType->insertItem( K3bMixedDoc::DATA_SECOND_SESSION,
+				    i18n("Data in second session (CD-Extra)"),
+				    i18n("<em>Blue book CD</em>"
+					 "<br>K3b will create a multisession CD with "
+					 "2 sessions. The first session will contain all "
+					 "audio tracks and the second session will contain "
+					 "a mode 2 form 1 data track."
+					 "<br>This mode is based on the <em>Blue book</em> "
+					 "standard (also known as <em>Extended Audio CD</em>, "
+					 "<em>CD-Extra</em>, or <em>CD Plus</em>) "
+					 "and has the advantage that a hifi audio "
+					 "CD player will only recognize the first session "
+					 "and ignore the second session with the data track."
+					 "<br>If the CD is intended to be used in a hifi audio CD player "
+					 "this is the recommended mode."
+					 "<br>Some older CD-ROMs may have problems reading "
+					 "a blue book CD since it is a multisession CD.") );
+  m_comboMixedModeType->insertItem( K3bMixedDoc::DATA_FIRST_TRACK,
+				    i18n("Data in first track"),
+				    i18n("K3b will write the data track before all "
+					 "audio tracks.") );
+  m_comboMixedModeType->insertItem( K3bMixedDoc::DATA_LAST_TRACK,
+				    i18n("Data in last track"),
+				    i18n("K3b will write the data track after all "
+					 "audio tracks.") );
+  m_comboMixedModeType->addGlobalWhatsThisText( QString(),
+						i18n("<b>Caution:</b> The last two modes should only be used for CDs that are unlikely to "
+						     "be played on a hifi audio CD player."
+						     "<br>It could lead to problems with some older "
+						     "hifi audio CD players that try to play the data track.") );
+				    
   QGridLayout* grid = new QGridLayout( w );
   grid->setMargin( marginHint() );
   grid->setSpacing( spacingHint() );
-  grid->addWidget( m_groupMixedType, 0, 0 );
+  grid->addWidget( groupMixedType, 0, 0 );
   grid->addWidget( groupDataMode, 1, 0 );
   grid->addWidget( groupNormalize, 2, 0 );
   grid->setRowStretch( 3, 1 );
 
-
-  addPage( w, i18n("Settings") );
-}
-
-
-void K3bMixedBurnDialog::createContextHelp()
-{
-  QToolTip::add( m_radioMixedTypeFirstTrack, i18n("First track will contain the data") );
-  QWhatsThis::add( m_radioMixedTypeFirstTrack, i18n("<p><b>Standard mixed mode CD 1</b>"
-						    "<p>K3b will write the data track before all "
-						    "audio tracks."
-						    "<p>This mode should only be used for CDs that are unlikely to "
-						    "be played on a hifi audio CD player."
-						    "<p><b>Caution:</b> It could lead to problems with some older "
-						    "hifi audio CD players that try to play the data track.") );
-
-  QToolTip::add( m_radioMixedTypeLastTrack, i18n("Last track will contain the data") );
-  QWhatsThis::add( m_radioMixedTypeLastTrack, i18n("<p><b>Standard mixed mode CD 2</b>"
-						   "<p>K3b will write the data track after all "
-						   "audio tracks."
-						    "<p>This mode should only be used for CDs that are unlikely to "
-						    "be played on a hifi audio CD player."
-						   "<p><b>Caution:</b> It could lead to problems with some older "
-						   "hifi audio CD players that try to play the data track.") );
-
-  QToolTip::add( m_radioMixedTypeSessions, i18n("The data will be written in a second session") );
-  QWhatsThis::add( m_radioMixedTypeSessions, i18n("<p><b>Blue book CD</b>"
-						  "<p>K3b will create a multisession CD with "
-						  "2 sessions. The first session will contain all "
-						  "audio tracks and the second session will contain "
-						  "a mode 2 form 1 data track."
-						  "<p>This mode is based on the <em>Blue book</em> "
-						  "standard (also known as <em>Extended Audio CD</em>, "
-						  "<em>CD-Extra</em>, or <em>CD Plus</em>) "
-						  "and has the advantage that a hifi audio "
-						  "CD player will only recognize the first session "
-						  "and ignore the second session with the data track."
-						  "<p>If the CD is intended to be used in a hifi audio CD player "
-						  "this is the recommended mode."
-						  "<p>Some older CD-ROMs may have problems reading "
-						  "a blue book CD since it is a multisession CD.") );
+  addPage( w, i18n("Misc") );
 }
 
 
@@ -194,12 +158,7 @@ void K3bMixedBurnDialog::saveSettings()
 {
   K3bProjectBurnDialog::saveSettings();
 
-  if( m_groupMixedType->selected() == m_radioMixedTypeLastTrack )
-    m_doc->setMixedType( K3bMixedDoc::DATA_LAST_TRACK );
-  else if( m_groupMixedType->selected() == m_radioMixedTypeSessions )
-    m_doc->setMixedType( K3bMixedDoc::DATA_SECOND_SESSION );
-  else
-    m_doc->setMixedType( K3bMixedDoc::DATA_FIRST_TRACK );
+  m_doc->setMixedType( (K3bMixedDoc::MixedType)m_comboMixedModeType->selectedValue() );
 
   m_cdtextWidget->save( m_doc->audioDoc() );
 
@@ -208,8 +167,6 @@ void K3bMixedBurnDialog::saveSettings()
   // save iso image settings
   K3bIsoOptions o = m_doc->dataDoc()->isoOptions();
   m_imageSettingsWidget->save( o );
-  m_advancedImageSettingsWidget->save( o );
-  m_volumeDescWidget->save( o );
   m_doc->dataDoc()->setIsoOptions( o );
 
   m_doc->dataDoc()->setDataMode( m_dataModeWidget->dataMode() );
@@ -228,27 +185,15 @@ void K3bMixedBurnDialog::readSettings()
   if( !m_doc->tempDir().isEmpty() )
     m_tempDirSelectionWidget->setTempPath( m_doc->tempDir() );
 
-  switch( m_doc->mixedType() ) {
-  case K3bMixedDoc::DATA_FIRST_TRACK:
-    m_radioMixedTypeFirstTrack->setChecked(true);
-    break;
-  case K3bMixedDoc::DATA_LAST_TRACK:
-    m_radioMixedTypeLastTrack->setChecked(true);
-    break;
-  case K3bMixedDoc::DATA_SECOND_SESSION:
-    m_radioMixedTypeSessions->setChecked(true);
-    break;
-  }
+  m_comboMixedModeType->setSelectedValue( m_doc->mixedType() );
 
   m_cdtextWidget->load( m_doc->audioDoc() );
 
   m_imageSettingsWidget->load( m_doc->dataDoc()->isoOptions() );
-  m_advancedImageSettingsWidget->load( m_doc->dataDoc()->isoOptions() );
-  m_volumeDescWidget->load( m_doc->dataDoc()->isoOptions() );
 
   m_dataModeWidget->setDataMode( m_doc->dataDoc()->dataMode() );
 
-  toggleAllOptions();
+  toggleAll();
 }
 
 
@@ -257,17 +202,15 @@ void K3bMixedBurnDialog::loadK3bDefaults()
   K3bProjectBurnDialog::loadK3bDefaults();
 
   m_cdtextWidget->setChecked( false );
-  m_checkNormalize->setChecked(false);
+  m_checkNormalize->setChecked( false );
 
-  m_radioMixedTypeSessions->setChecked(true);
+  m_comboMixedModeType->setSelectedValue( K3bMixedDoc::DATA_SECOND_SESSION );
 
   m_dataModeWidget->setDataMode( K3b::DATA_MODE_AUTO );
 
   m_imageSettingsWidget->load( K3bIsoOptions::defaults() );
-  m_advancedImageSettingsWidget->load( K3bIsoOptions::defaults() );
-  m_volumeDescWidget->load( K3bIsoOptions::defaults() );
 
-  toggleAllOptions();
+  toggleAll();
 }
 
 
@@ -280,20 +223,18 @@ void K3bMixedBurnDialog::loadUserDefaults( KConfigBase* c )
 
   // load mixed type
   if( c->readEntry( "mixed_type" ) == "last_track" )
-    m_radioMixedTypeLastTrack->setChecked(true);
+    m_comboMixedModeType->setSelectedValue( K3bMixedDoc::DATA_LAST_TRACK );
   else if( c->readEntry( "mixed_type" ) == "first_track" )
-    m_radioMixedTypeFirstTrack->setChecked(true);
+    m_comboMixedModeType->setSelectedValue( K3bMixedDoc::DATA_FIRST_TRACK );
   else
-    m_radioMixedTypeSessions->setChecked(true);
+    m_comboMixedModeType->setSelectedValue( K3bMixedDoc::DATA_SECOND_SESSION );
 
   m_dataModeWidget->loadConfig(c);
 
   K3bIsoOptions o = K3bIsoOptions::load( c );
   m_imageSettingsWidget->load( o );
-  m_advancedImageSettingsWidget->load( o );
-  m_volumeDescWidget->load( o );
 
-  toggleAllOptions();
+  toggleAll();
 }
 
 
@@ -305,19 +246,21 @@ void K3bMixedBurnDialog::saveUserDefaults( KConfigBase* c )
   c->writeEntry( "normalize", m_checkNormalize->isChecked() );
 
   // save mixed type
-  if( m_groupMixedType->selected() == m_radioMixedTypeLastTrack )
+  switch( m_comboMixedModeType->selectedValue() ) {
+  case K3bMixedDoc::DATA_LAST_TRACK:
    c->writeEntry( "mixed_type", "last_track" );
-  else if( m_groupMixedType->selected() == m_radioMixedTypeSessions )
-   c->writeEntry( "mixed_type", "second_session" );
-  else
+   break;
+  case K3bMixedDoc::DATA_FIRST_TRACK:
     c->writeEntry( "mixed_type", "first_track" );
+    break;
+  default:
+   c->writeEntry( "mixed_type", "second_session" );
+  }
 
   m_dataModeWidget->saveConfig(c);
 
   K3bIsoOptions o;
   m_imageSettingsWidget->save( o );
-  m_advancedImageSettingsWidget->save( o );
-  m_volumeDescWidget->save( o );
   o.save( c );
 
   if( m_tempDirSelectionWidget->isEnabled() ) {
@@ -326,15 +269,14 @@ void K3bMixedBurnDialog::saveUserDefaults( KConfigBase* c )
 }
 
 
-void K3bMixedBurnDialog::toggleAllOptions()
+void K3bMixedBurnDialog::toggleAll()
 {
-  K3bProjectBurnDialog::toggleAllOptions();
+  K3bProjectBurnDialog::toggleAll();
 
   bool cdrecordOnTheFly = false;
   bool cdrecordCdText = false;
   if ( k3bcore->externalBinManager()->binObject("cdrecord") ) {
-    cdrecordOnTheFly = k3bcore->externalBinManager()->binObject("cdrecord")->version
-      >= K3bVersion( 2, 1, -1, "a13" );
+    cdrecordOnTheFly = k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "audio-stdin" );
     cdrecordCdText = k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "cdtext" );
   }
 
@@ -348,21 +290,62 @@ void K3bMixedBurnDialog::toggleAllOptions()
     cdText = cdrecordCdText;
   }
 
-  m_checkOnTheFly->setEnabled( !m_checkOnlyCreateImage->isChecked() && 
-			       onTheFly && 
-			       !m_checkNormalize->isChecked() );
-  if( !onTheFly || m_checkNormalize->isChecked() )
-    m_checkOnTheFly->setChecked( false );
+  m_checkCacheImage->setEnabled( !m_checkOnlyCreateImage->isChecked() && 
+				 onTheFly );
+  if( !onTheFly )
+    m_checkCacheImage->setChecked( true );
 
   m_cdtextWidget->setEnabled( !m_checkOnlyCreateImage->isChecked() &&
 			      cdText && 
 			      m_writingModeWidget->writingMode() != K3b::TAO );
   if( !cdText || m_writingModeWidget->writingMode() == K3b::TAO  )
     m_cdtextWidget->setChecked( false );
+}
 
-  // we are not able to normalize in on-the-fly mode
-  m_checkNormalize->setDisabled( !k3bcore->externalBinManager()->foundBin( "normalize" ) ||
-				 ( m_checkOnTheFly->isChecked() && !m_checkOnlyCreateImage->isChecked() ) );
+
+void K3bMixedBurnDialog::slotNormalizeToggled( bool on )
+{
+  if( on ) {
+    // we are not able to normalize in on-the-fly mode
+    if( !k3bcore->externalBinManager()->foundBin( "normalize-audio" ) ) {
+      KMessageBox::sorry( this, i18n("<p><b>External program <em>normalize-audio</em> is not installed.</b>"
+				     "<p>K3b uses <em>normalize-audio</em> (http://www1.cs.columbia.edu/~cvaill/normalize/) "
+				     "to normalize audio tracks. In order to "
+				     "use this functionality, please install it first.") );
+      m_checkNormalize->setChecked( false );
+    }
+    else if( !m_checkCacheImage->isChecked() && !m_checkOnlyCreateImage->isChecked() ) {
+      if( KMessageBox::warningYesNo( this, i18n("<p>K3b is not able to normalize audio tracks when burning on-the-fly. "
+						"The external program used for this task only supports normalizing a set "
+						"of audio files."),
+				     QString::null,
+				     i18n("Disable normalization"),
+				     i18n("Disable on-the-fly burning"),
+				     "audioProjectNormalizeOrOnTheFly" ) == KMessageBox::Yes )
+	m_checkNormalize->setChecked( false );
+      else
+	m_checkCacheImage->setChecked( true );
+    }
+  }
+}
+
+
+void K3bMixedBurnDialog::slotCacheImageToggled( bool on )
+{
+  if( on ) {
+    if( m_checkNormalize->isChecked() ) {
+      if( KMessageBox::warningYesNo( this, i18n("<p>K3b is not able to normalize audio tracks when burning on-the-fly. "
+						"The external program used for this task only supports normalizing a set "
+						"of audio files."),
+				     QString::null,
+				     i18n("Disable normalization"),
+				     i18n("Disable on-the-fly burning"),
+				     "audioProjectNormalizeOrOnTheFly" ) == KMessageBox::Yes )
+	m_checkNormalize->setChecked( false );
+      else
+	m_checkCacheImage->setChecked( true );
+    }
+  }
 }
 
 
